# Source Generated with Decompyle++
# File: in.pyc (Python 2.5)

import warnings
warnings.filterwarnings('ignore', 'apt API not stable yet', FutureWarning)
import apt
import apt_pkg
import sys
import os
import subprocess
import logging
import re
import statvfs
import shutil
import glob
import time
import copy
import ConfigParser
from stat import *
from string import Template
import DistUpgradeView
from DistUpgradeConfigParser import DistUpgradeConfig
from DistUpgradeFetcherCore import country_mirror
from DistUpgradeQuirks import DistUpgradeQuirks
from DistUpgradeAptCdrom import AptCdrom
from sourceslist import SourcesList, SourceEntry, is_mirror
from distro import Distribution, get_distro, NoDistroTemplateException
from DistUpgradeGettext import gettext as _
from DistUpgradeGettext import ngettext
import gettext
from DistUpgradeCache import *
from DistUpgradeApport import *
KERNEL_INITRD_SIZE = 12 * 1024 * 1024

class NoBackportsFoundException(Exception):
    pass


class DistUpgradeController(object):
    ''' this is the controller that does most of the work '''
    
    def __init__(self, distUpgradeView, options = None, datadir = None):
        localedir = '/usr/share/locale/update-manager/'
        if datadir == None:
            datadir = os.getcwd()
            localedir = os.path.join(datadir, 'mo')
            gladedir = datadir
        
        self.datadir = datadir
        self.options = options
        gettext.bindtextdomain('update-manager', localedir)
        gettext.textdomain('update-manager')
        logging.debug("Using '%s' view" % distUpgradeView.__class__.__name__)
        self._view = distUpgradeView
        self._view.updateStatus(_('Reading cache'))
        self.cache = None
        if not (self.options) or self.options.withNetwork == None:
            self.useNetwork = True
        else:
            self.useNetwork = self.options.withNetwork
        if options:
            cdrompath = options.cdromPath
        else:
            cdrompath = None
        self.aptcdrom = AptCdrom(distUpgradeView, cdrompath)
        self.config = DistUpgradeConfig(datadir)
        self.sources_backup_ext = '.' + self.config.get('Files', 'BackupExt')
        self.config.add_section('Options')
        self.config.set('Options', 'withNetwork', str(self.useNetwork))
        self.fromDist = self.config.get('Sources', 'From')
        self.toDist = self.config.get('Sources', 'To')
        self.origin = self.config.get('Sources', 'ValidOrigin')
        self.arch = apt_pkg.Config.Find('APT::Architecture')
        if not os.environ.has_key('RELEASE_UPGRADE_NO_FORCE_OVERWRITE'):
            logging.debug('enable dpkg --force-overwrite')
            apt_pkg.Config.Set('DPkg::Options::', '--force-overwrite')
        
        self._partialUpgrade = False
        self.quirks = DistUpgradeQuirks(self, self.config)
        os.environ['RELEASE_UPGRADE_IN_PROGRESS'] = '1'
        os.environ['PYCENTRAL_NO_DPKG_QUERY'] = '1'
        os.environ['PATH'] = '%s:%s' % (os.getcwd() + '/imported', os.environ['PATH'])
        maxRetries = self.config.getint('Network', 'MaxRetries')
        apt_pkg.Config.Set('Acquire::Retries', str(maxRetries))
        apt_pkg.Config.Set('Dpkg::MaxArgs', str(65536))
        apt_pkg.Config.Set('Dpkg::MaxArgBytes', str(131072))
        apt_pkg.Config.Set('Acquire::http::Timeout', '20')
        apt_pkg.Config.Set('Acquire::ftp::Timeout', '20')
        self.forced_obsoletes = self.config.getlist('Distro', 'ForcedObsoletes')
        self.valid_mirrors = self.config.getListFromFile('Sources', 'ValidMirrors')

    
    def openCache(self, lock = True):
        logging.debug('openCache()')
        if self.cache is not None:
            self.cache.releaseLock()
            self.cache.unlockListsDir()
        
        
        try:
            self.cache = MyCache(self.config, self._view, self.quirks, self._view.getOpCacheProgress(), lock)
        except CacheExceptionDpkgInterrupted:
            e = None
            logging.warning('dpkg interrupted, calling dpkg --configure -a')
            self._view.getTerminal().call([
                'dpkg',
                '--configure',
                '-a'])
            self.cache = MyCache(self.config, self._view, self.quirks, self._view.getOpCacheProgress())
        except CacheExceptionLockingFailed:
            e = None
            logging.error('Cache can not be locked (%s)' % e)
            self._view.error(_('Unable to get exclusive lock'), _('This usually means that another package management application (like apt-get or aptitude) already running. Please close that application first.'))
            sys.exit(1)

        logging.debug('/openCache()')

    
    def _sshMagic(self):
        ''' this will check for server mode and if we run over ssh.
            if this is the case, we will ask and spawn a additional
            daemon (to be sure we have a spare one around in case
            of trouble)
        '''
        pidfile = os.path.join('/var/run/release-upgrader-sshd.pid')
        if not os.path.exists(pidfile):
            if os.environ.has_key('SSH_CONNECTION') or os.environ.has_key('SSH_TTY'):
                port = 9004
                res = self._view.askYesNoQuestion(_('Continue running under SSH?'), _("This session appears to be running under ssh. It is not recommended to perform a upgrade over ssh currently because in case of failure it is harder to recover.\n\nIf you continue, a additional ssh daemon will be started at port '%s'.\nDo you want to continue?") % port)
                if res == False:
                    sys.exit(1)
                
                res = subprocess.call([
                    '/usr/sbin/sshd',
                    '-o',
                    'PidFile=%s' % pidfile,
                    '-p',
                    str(port)])
                if res == 0:
                    self._view.information(_('Starting additional sshd'), _("To make recovery in case of failure easier, an additional sshd will be started on port '%s'. If anything goes wrong with the running ssh you can still connect to the additional one.\n") % port)
                
            

    
    def _tryUpdateSelf(self):
        ''' this is a helper that is run if we are started from a CD
            and we have network - we will then try to fetch a update
            of ourself
        '''
        MetaReleaseCore = MetaReleaseCore
        import MetaRelease
        DistUpgradeFetcherSelf = DistUpgradeFetcherSelf
        import DistUpgradeFetcherSelf
        forceLTS = False
        if self.release == 'dapper' or self.release == 'hardy':
            forceLTS = True
        
        m = MetaReleaseCore(useDevelopmentRelease = False, forceLTS = forceLTS)
        while m.downloading:
            self._view.processEvents()
            time.sleep(0.1)
        if m.new_dist is None:
            logging.error('No new dist found')
            return False
        
        progress = self._view.getFetchProgress()
        fetcher = DistUpgradeFetcherSelf(new_dist = m.new_dist, progress = progress, options = self.options, view = self._view)
        fetcher.run()

    
    def _pythonSymlinkCheck(self):
        ''' sanity check that /usr/bin/python points to the default
            python version. Users tend to modify this symlink, which
            breaks stuff in obscure ways (Ubuntu #75557).
        '''
        logging.debug('_pythonSymlinkCheck run')
        SafeConfigParser = SafeConfigParser
        NoOptionError = NoOptionError
        import ConfigParser
        if os.path.exists('/usr/share/python/debian_defaults'):
            config = SafeConfigParser()
            config.readfp(file('/usr/share/python/debian_defaults'))
            
            try:
                expected_default = config.get('DEFAULT', 'default-version')
            except NoOptionError:
                logging.debug("no default version for python found in '%s'" % config)
                return False

            
            try:
                fs_default_version = os.readlink('/usr/bin/python')
            except OSError:
                e = None
                logging.error('os.readlink failed (%s)' % e)
                return False

            if fs_default_version not in (expected_default, os.path.join('/usr/bin', expected_default)):
                logging.debug("python symlink points to: '%s', but expected is '%s' or '%s'" % (fs_default_version, expected_default, os.path.join('/usr/bin', expected_default)))
                return False
            
        
        return True

    
    def prepare(self):
        ''' initial cache opening, sanity checking, network checking '''
        self.release = release = subprocess.Popen([
            'lsb_release',
            '-c',
            '-s'], stdout = subprocess.PIPE).communicate()[0].strip()
        logging.debug("lsb-release: '%s'" % release)
        if not release == self.fromDist or release == self.toDist:
            logging.error("Bad upgrade: '%s' != '%s' " % (release, self.fromDist))
            self._view.error(_('Can not upgrade'), _("An upgrade from '%s' to '%s' is not supported with this tool." % (release, self.toDist)))
            sys.exit(1)
        
        if self.options and self.options.havePrerequists:
            backportsdir = os.getcwd() + '/backports'
            logging.info("using backports in '%s' " % backportsdir)
            logging.debug('have: %s' % glob.glob(backportsdir + '/*.udeb'))
            if os.path.exists(backportsdir + '/usr/bin/dpkg'):
                apt_pkg.Config.Set('Dir::Bin::dpkg', backportsdir + '/usr/bin/dpkg')
            
            if os.path.exists(backportsdir + '/usr/lib/apt/methods'):
                apt_pkg.Config.Set('Dir::Bin::methods', backportsdir + '/usr/lib/apt/methods')
            
            conf = backportsdir + '/etc/apt/apt.conf.d/01ubuntu'
            if os.path.exists(conf):
                logging.debug("adding config '%s'" % conf)
                apt_pkg.ReadConfigFile(apt_pkg.Config, conf)
            
        
        self._sshMagic()
        if not self._pythonSymlinkCheck():
            logging.error('pythonSymlinkCheck() failed, aborting')
            self._view.error(_('Can not upgrade'), _("Your python install is corrupted. Please fix the '/usr/bin/python' symlink."))
            sys.exit(1)
        
        
        try:
            self.openCache()
        except SystemError:
            e = None
            logging.error("openCache() failed: '%s'" % e)
            return False

        if not self.cache.sanityCheck(self._view):
            return False
        
        self.serverMode = self.cache.needServerMode()
        if self.serverMode:
            os.environ['RELEASE_UPGRADE_MODE'] = 'server'
        else:
            os.environ['RELEASE_UPGRADE_MODE'] = 'desktop'
        if not self.checkViewDepends():
            logging.error('checkViewDepends() failed')
            return False
        
        if os.path.exists('/usr/bin/debsig-verify'):
            logging.error('debsig-verify is installed')
            self._view.error(_("Package 'debsig-verify' is installed"), _("The upgrade can not continue with that package installed.\nPlease remove it with synaptic or 'apt-get remove debsig-verify' first and run the upgrade again."))
            self.abort()
        
        if self.aptcdrom and self.options and self.options.withNetwork == None:
            res = self._view.askYesNoQuestion(_('Include latest updates from the Internet?'), _("The upgrade system can use the internet to automatically download the latest updates and install them during the upgrade.  If you have a network connection this is highly recommended.\n\nThe upgrade will take longer, but when it is complete, your system will be fully up to date.  You can choose not to do this, but you should install the latest updates soon after upgrading.\nIf you answer 'no' here, the network is not used at all."), 'Yes')
            self.useNetwork = res
            self.config.set('Options', 'withNetwork', str(self.useNetwork))
            logging.debug("useNetwork: '%s' (selected by user)" % res)
            if res:
                self._tryUpdateSelf()
            
        
        return True

    
    def rewriteSourcesList(self, mirror_check = True):
        logging.debug('rewriteSourcesList()')
        if mirror_check == True and self.useNetwork:
            for pkgname in self.config.getlist('Distro', 'BaseMetaPkgs'):
                if (not self.cache.has_key(pkgname) or len(self.cache[pkgname].candidateOrigin) == 0 or len(self.cache[pkgname].candidateOrigin) == 1) and self.cache[pkgname].candidateOrigin[0].archive == 'now':
                    logging.debug("BaseMetaPkg '%s' has no candidateOrigin" % pkgname)
                    
                    try:
                        distro = get_distro()
                        distro.get_sources(self.sources)
                        distro.enable_component('main')
                    except NoDistroTemplateException:
                        e = None
                        logging.warning('get_distro().enable_component("man") failed, overwriting sources.list instead as last resort')
                        s = 'deb http://archive.ubuntu.com/ubuntu %s main' % self.toDist
                        open('/etc/apt/sources.list', 'w').write(s)

                    break
                    continue
            
        
        fromDists = [
            self.fromDist,
            self.fromDist + '-security',
            self.fromDist + '-updates',
            self.fromDist + '-proposed',
            self.fromDist + '-backports']
        toDists = [
            self.toDist,
            self.toDist + '-security',
            self.toDist + '-updates',
            self.toDist + '-proposed',
            self.toDist + '-backports']
        self.sources_disabled = False
        foundToDist = False
        for entry in self.sources.list[:]:
            if entry.invalid or entry.disabled:
                continue
            
            if entry.uri.startswith('cdrom:') and entry.dist == self.fromDist:
                entry.disabled = True
                continue
            elif entry.uri.startswith('cdrom:'):
                continue
            
            cdist = '%s-commercial' % self.fromDist
            if not (entry.disabled) and entry.uri.startswith('http://archive.canonical.com') and entry.dist == cdist:
                entry.dist = self.toDist
                entry.comps = [
                    'partner']
                logging.debug("transitioned commercial to '%s' " % entry)
                continue
            
            if not (entry.disabled) and entry.uri.startswith('http://landscape.canonical.com/packages/%s' % self.fromDist):
                logging.debug('commenting landscape.canonical.com out')
                entry.disabled = True
                continue
            
            if not (entry.disabled) and entry.uri.startswith('http://old-releases.ubuntu.com/ubuntu'):
                entry.uri = 'http://archive.ubuntu.com/ubuntu'
                logging.debug("transitioning old-releases.ubuntu.com to '%s' " % entry)
                continue
            
            logging.debug("examining: '%s'" % entry)
            validMirror = self.isMirror(entry.uri)
            if validMirror or not mirror_check:
                validMirror = True
                validTo = True
                if entry.disabled and entry.type == 'deb-src' and entry.uri.startswith('http://security.ubuntu.com') or entry.uri.startswith('http://archive.canonical.com'):
                    validTo = False
                
                if entry.dist in toDists:
                    logging.debug("entry '%s' is already set to new dist" % entry)
                    foundToDist |= validTo
                elif entry.dist in fromDists:
                    foundToDist |= validTo
                    entry.dist = toDists[fromDists.index(entry.dist)]
                    logging.debug("entry '%s' updated to new dist" % entry)
                else:
                    entry.disabled = True
                    self.sources_disabled = True
                    logging.debug("entry '%s' was disabled (unknown dist)" % entry)
                if entry.type == 'deb' and 'ports.ubuntu.com' not in entry.uri:
                    pass
                None if self.arch == 'powerpc' or self.arch == 'sparc' else self.arch == 'sparc'
            
            if not validMirror:
                entry.disabled = True
                self.sources_disabled = True
                logging.debug("entry '%s' was disabled (unknown mirror)" % entry)
                continue
        
        return foundToDist

    
    def updateSourcesList(self):
        logging.debug('updateSourcesList()')
        self.sources = SourcesList(matcherPath = '.')
        if not self.rewriteSourcesList(mirror_check = True):
            logging.error('No valid mirror found')
            res = self._view.askYesNoQuestion(_('No valid mirror found'), _("While scanning your repository information no mirror entry for the upgrade was found.This can happen if you run a internal mirror or if the mirror information is out of date.\n\nDo you want to rewrite your 'sources.list' file anyway? If you choose 'Yes' here it will update all '%s' to '%s' entries.\nIf you select 'no' the update will cancel.") % (self.fromDist, self.toDist))
            if res:
                self.sources = SourcesList(matcherPath = '.')
                if not self.rewriteSourcesList(mirror_check = False) and self.useNetwork:
                    prim = _('Generate default sources?')
                    secon = _("After scanning your 'sources.list' no valid entry for '%s' was found.\n\nShould default entries for '%s' be added? If you select 'No', the update will cancel.") % (self.fromDist, self.toDist)
                    if not self._view.askYesNoQuestion(prim, secon):
                        self.abort()
                    
                    uri = 'http://archive.ubuntu.com/ubuntu'
                    comps = [
                        'main',
                        'restricted']
                    self.sources.add('deb', uri, self.toDist, comps)
                    self.sources.add('deb', uri, self.toDist + '-updates', comps)
                    self.sources.add('deb', 'http://security.ubuntu.com/ubuntu/', self.toDist + '-security', comps)
                
            else:
                self.abort()
        
        self.sources.backup(self.sources_backup_ext)
        self.sources.save()
        
        try:
            sourceslist = apt_pkg.GetPkgSourceList()
            sourceslist.ReadMainList()
        except SystemError:
            logging.error('Repository information invalid after updating (we broke it!)')
            self._view.error(_('Repository information invalid'), _('Upgrading the repository information resulted in a invalid file. Please report this as a bug.'))
            return False

        if self.sources_disabled:
            self._view.information(_('Third party sources disabled'), _("Some third party entries in your sources.list were disabled. You can re-enable them after the upgrade with the 'software-properties' tool or your package manager."))
        
        return True

    
    def _logChanges(self):
        logging.debug('About to apply the following changes')
        inst = []
        up = []
        rm = []
        held = []
        keep = []
        for pkg in self.cache:
            if pkg.markedInstall:
                inst.append(pkg.name)
                continue
            if pkg.markedUpgrade:
                up.append(pkg.name)
                continue
            if pkg.markedDelete:
                rm.append(pkg.name)
                continue
            if pkg.isInstalled and pkg.isUpgradable:
                held.append(pkg.name)
                continue
            if pkg.isInstalled and pkg.markedKeep:
                keep.append(pkg.name)
                continue
        
        logging.debug('Keep at same version: %s' % ' '.join(keep))
        logging.debug('Upgradable, but held- back: %s' % ' '.join(held))
        logging.debug('Remove: %s' % ' '.join(rm))
        logging.debug('Install: %s' % ' '.join(inst))
        logging.debug('Upgrade: %s' % ' '.join(up))

    
    def doPostInitialUpdate(self):
        logging.debug('doPostInitialUpdate')
        self.quirks.run('PostInitialUpdate')
        if len(self.cache.reqReinstallPkgs) > 0:
            logging.warning('packages in reqReinstall state, trying to fix')
            self.cache.fixReqReinst(self._view)
            self.openCache()
        
        if len(self.cache.reqReinstallPkgs) > 0:
            reqreinst = self.cache.reqReinstallPkgs
            header = ngettext('Package in inconsistent state', 'Packages in inconsistent state', len(reqreinst))
            summary = ngettext("The package '%s' is in an inconsistent state and needs to be reinstalled, but no archive can be found for it. Please reinstall the package manually or remove it from the system.", "The packages '%s' are in an inconsistent state and need to be reinstalled, but no archive can be found for it.Please reinstall the packages manually or remove them from the system.", len(reqreinst)) % ', '.join(reqreinst)
            self._view.error(header, summary)
            return False
        
        self.obsolete_pkgs = self.cache._getObsoletesPkgs()
        self.foreign_pkgs = self.cache._getForeignPkgs(self.origin, self.fromDist, self.toDist)
        if self.serverMode:
            self.tasks = self.cache.installedTasks
        
        logging.debug('Foreign: %s' % ' '.join(self.foreign_pkgs))
        logging.debug('Obsolete: %s' % ' '.join(self.obsolete_pkgs))
        return True

    
    def doUpdate(self, showErrors = True, forceRetries = None):
        logging.debug('running doUpdate() (showErrors=%s)' % showErrors)
        if not self.useNetwork:
            logging.debug('doUpdate() will not use the network because self.useNetwork==false')
            return True
        
        self.cache._list.ReadMainList()
        progress = self._view.getFetchProgress()
        currentRetry = 0
        if forceRetries is not None:
            maxRetries = forceRetries
        else:
            maxRetries = self.config.getint('Network', 'MaxRetries')
        while currentRetry < maxRetries:
            
            try:
                res = self.cache.update(progress)
            except (SystemError, IOError):
                e = None
                logging.error("IOError/SystemError in cache.update(): '%s'. Retrying (currentRetry: %s)" % (e, currentRetry))
                currentRetry += 1
                continue

            return True
        logging.error('doUpdate() failed completely')
        if showErrors:
            self._view.error(_('Error during update'), _('A problem occurred during the update. This is usually some sort of network problem, please check your network connection and retry.'), '%s' % e)
        
        return False

    
    def _checkFreeSpace(self):
        ''' this checks if we have enough free space on /var and /usr'''
        err_sum = _('Not enough free disk space')
        err_long = _("The upgrade aborts now. The upgrade needs a total of %s free space on disk '%s'. Please free at least an additional %s of disk space on '%s'. Empty your trash and remove temporary packages of former installations using 'sudo apt-get clean'.")
        
        class FreeSpace(object):
            ''' helper class that represents the free space on each mounted fs '''
            
            def __init__(self, initialFree):
                self.free = initialFree
                self.need = 0


        
        def make_fs_id(d):
            """ return 'id' of a directory so that directories on the
                same filesystem get the same id (simply the mount_point)
            """
            for mount_point in mounted:
                if d.startswith(mount_point):
                    return mount_point
                    continue
            
            return '/'

        mounted = []
        mnt_map = { }
        fs_free = { }
        for line in open('/proc/mounts'):
            
            try:
                (what, where, fs, options, a, b) = line.split()
            except ValueError:
                (None,)
                e = (None,)
                logging.debug("line '%s' in /proc/mounts not understood (%s)" % (line, e))
                continue
            except:
                (None,)

            if where not in mounted:
                mounted.append(where)
                continue
            (None,)
        
        mounted.sort(cmp = (lambda a, b: cmp(len(a), len(b))), reverse = True)
        archivedir = apt_pkg.Config.FindDir('Dir::Cache::archives')
        for d in [
            '/',
            '/usr',
            '/var',
            '/boot',
            archivedir,
            '/home']:
            d = os.path.realpath(d)
            fs_id = make_fs_id(d)
            st = os.statvfs(d)
            free = st[statvfs.F_BAVAIL] * st[statvfs.F_FRSIZE]
            if fs_id in mnt_map:
                logging.debug('Dir %s mounted on %s' % (d, mnt_map[fs_id]))
                fs_free[d] = fs_free[mnt_map[fs_id]]
                continue
            logging.debug('Free space on %s: %s' % (d, free))
            mnt_map[fs_id] = d
            fs_free[d] = FreeSpace(free)
        
        del mnt_map
        logging.debug("fs_free contains: '%s'" % fs_free)
        space_in_boot = 0
        for pkg in self.cache:
            if re.match('^linux-(image|image-debug)-[0-9.]*-.*', pkg.name):
                if pkg.markedInstall:
                    logging.debug('%s (new-install) added with %s to boot space' % (pkg.name, KERNEL_INITRD_SIZE))
                    space_in_boot += KERNEL_INITRD_SIZE
                elif pkg.markedUpgrade or pkg.isInstalled:
                    logging.debug('%s (upgrade|installed) added with %s to boot space' % (pkg.name, KERNEL_INITRD_SIZE))
                    space_in_boot += KERNEL_INITRD_SIZE
                
            pkg.isInstalled
        
        for dir, size in [
            (archivedir, self.cache.requiredDownload),
            ('/usr', self.cache.additionalRequiredSpace),
            ('/usr', 52428800),
            ('/boot', space_in_boot),
            ('/', 10485760)]:
            dir = os.path.realpath(dir)
            logging.debug("dir '%s' needs '%s' of '%s' (%f)" % (dir, size, fs_free[dir], fs_free[dir].free))
            fs_free[dir].free -= size
            fs_free[dir].need += size
            if fs_free[dir].free < 0:
                free_at_least = apt_pkg.SizeToStr(float(abs(fs_free[dir].free) + 1))
                logging.error('not enough free space on %s (missing %s)' % (dir, free_at_least))
                self._view.error(err_sum, err_long % (apt_pkg.SizeToStr(fs_free[dir].need), make_fs_id(dir), free_at_least, make_fs_id(dir)))
                return False
                continue
            fs_free[dir]
        
        return True

    
    def askDistUpgrade(self):
        demotions = set()
        demotions_file = self.config.get('Distro', 'Demotions')
        if os.path.exists(demotions_file):
            (map,)((lambda pkgname: demotions.add(pkgname.strip())), filter((lambda line: not line.startswith('#')), open(demotions_file).readlines()))
        
        self.installed_demotions = _[1]
        if not self.cache.distUpgrade(self._view, self.serverMode, self._partialUpgrade):
            return False
        
        if self.serverMode:
            if not self.cache.installTasks(self.tasks):
                return False
            
        
        changes = self.cache.getChanges()
        self._logChanges()
        if not self._checkFreeSpace():
            return False
        
        res = self._view.confirmChanges(_('Do you want to start the upgrade?'), changes, self.cache.requiredDownload)
        return res

    
    def _disableAptCronJob(self):
        self._aptCronJobPerms = 493
        if os.path.exists('/etc/cron.daily/apt'):
            self._aptCronJobPerms = os.stat('/etc/cron.daily/apt')[ST_MODE]
            logging.debug('disabling apt cron job (%s)' % oct(self._aptCronJobPerms))
            os.chmod('/etc/cron.daily/apt', 420)
        

    
    def _enableAptCronJob(self):
        if os.path.exists('/etc/cron.daily/apt'):
            logging.debug('enabling apt cron job')
            os.chmod('/etc/cron.daily/apt', self._aptCronJobPerms)
        

    
    def doDistUpgradeFetching(self):
        self._disableAptCronJob()
        currentRetry = 0
        fprogress = self._view.getFetchProgress()
        iprogress = self._view.getInstallProgress(self.cache)
        maxRetries = self.config.getint('Network', 'MaxRetries')
        while currentRetry < maxRetries:
            
            try:
                pm = apt_pkg.GetPackageManager(self.cache._depcache)
                fetcher = apt_pkg.GetAcquire(fprogress)
                res = self.cache._fetchArchives(fetcher, pm)
            except IOError:
                e = None
                logging.error("IOError in cache.commit(): '%s'. Retrying (currentTry: %s)" % (e, currentRetry))
                currentRetry += 1
                continue

            return True
        logging.error('giving up on fetching after maximum retries')
        self._view.error(_('Could not download the upgrades'), _('The upgrade aborts now. Please check your Internet connection or installation media and try again. All files downloaded so far are kept.'), '%s' % e)
        self._enableAptCronJob()
        self.abort()

    
    def doDistUpgrade(self):
        currentRetry = 0
        fprogress = self._view.getFetchProgress()
        iprogress = self._view.getInstallProgress(self.cache)
        maxRetries = self.config.getint('Network', 'MaxRetries')
        while currentRetry < maxRetries:
            
            try:
                res = self.cache.commit(fprogress, iprogress)
            except SystemError:
                e = None
                logging.error('SystemError from cache.commit(): %s' % e)
                if iprogress.pkg_failures == 0:
                    logging.warning('cache.commit() error and pkg_failures == 0, generate a report against update-manager to investigate')
                    errormsg = 'SystemError in cache.commit(): %s' % e
                    apport_pkgfailure('update-manager', errormsg)
                
                msg = _('The upgrade aborts now. Your system could be in an unusable state. A recovery will run now (dpkg --configure -a).')
                if not self._partialUpgrade:
                    if not run_apport():
                        msg += _("\n\nPlease report this bug against the 'update-manager' package and include the files in /var/log/dist-upgrade/ in the bugreport.\n%s" % e)
                    
                
                self._view.error(_('Could not install the upgrades'), msg)
                self._view.getTerminal().call([
                    'dpkg',
                    '--configure',
                    '-a'])
                self._enableAptCronJob()
                return False
            except IOError:
                e = None
                logging.error("IOError in cache.commit(): '%s'. Retrying (currentTry: %s)" % (e, currentRetry))
                currentRetry += 1
                continue

            self._enableAptCronJob()
            return True
        logging.error('giving up on fetching after maximum retries')
        self._view.error(_('Could not download the upgrades'), _('The upgrade aborts now. Please check your Internet connection or installation media and try again. '), '%s' % e)
        self.abort()

    
    def doPostUpgrade(self):
        self.openCache()
        if not self._partialUpgrade:
            self.quirks.run('PostUpgrade')
        
        now_obsolete = self.cache._getObsoletesPkgs()
        now_foreign = self.cache._getForeignPkgs(self.origin, self.fromDist, self.toDist)
        logging.debug('Obsolete: %s' % ' '.join(now_obsolete))
        logging.debug('Foreign: %s' % ' '.join(now_foreign))
        if not self.config.getWithDefault('Distro', 'RemoveObsoletes', True):
            logging.debug('Skipping obsolete Removal')
            return True
        
        for pkg in self.config.getlist('Distro', 'MetaPkgs'):
            if self.cache.has_key(pkg) and self.cache[pkg].isInstalled:
                self.forced_obsoletes.extend(self.config.getlist(pkg, 'ForcedObsoletes'))
                continue
        
        self.forced_obsoletes.extend(self.cache.identifyObsoleteKernels())
        logging.debug('forced_obsoletes: %s', self.forced_obsoletes)
        if self.useNetwork:
            remove_candidates = now_obsolete - self.obsolete_pkgs
        else:
            remove_candidates = set(self.installed_demotions)
        remove_candidates |= set(self.forced_obsoletes)
        unused_dependencies = self.cache._getUnusedDependencies()
        logging.debug('Unused dependencies: %s' % ' '.join(unused_dependencies))
        remove_candidates |= set(unused_dependencies)
        if not self.config.getWithDefault('Distro', 'RemoveObsoletes', True):
            logging.debug('Skipping RemoveObsoletes as stated in the config')
            remove_candidates = set()
        
        logging.debug("remove_candidates: '%s'" % remove_candidates)
        logging.debug('Start checking for obsolete pkgs')
        for pkgname in remove_candidates:
            if pkgname not in self.foreign_pkgs:
                self._view.processEvents()
                if not self.cache.tryMarkObsoleteForRemoval(pkgname, remove_candidates, self.foreign_pkgs):
                    logging.debug("'%s' scheduled for remove but not safe to remove, skipping", pkgname)
                
            self.cache.tryMarkObsoleteForRemoval(pkgname, remove_candidates, self.foreign_pkgs)
        
        logging.debug('Finish checking for obsolete pkgs')
        changes = self.cache.getChanges()
        ' '.join([] % []([ pkg.name for pkg in changes ]))
        summary = _('Remove obsolete packages?')
        actions = [
            _('_Keep'),
            _('_Remove')]
        if len(changes) > 0 and self._view.confirmChanges(summary, changes, 0, actions, False):
            fprogress = self._view.getFetchProgress()
            iprogress = self._view.getInstallProgress(self.cache)
            
            try:
                res = self.cache.commit(fprogress, iprogress)
            except (SystemError, IOError):
                'The following packages are remove candidates: %s'
                e = 'The following packages are remove candidates: %s'
                logging.debug
                logging.error('cache.commit() in doPostUpgrade() failed: %s' % e)
                self._view.error(_('Error during commit'), _('A problem occurred during the clean-up. Please see the below message for more information. '), '%s' % e)
            except:
                'The following packages are remove candidates: %s'<EXCEPTION MATCH>(SystemError, IOError)
            

        'The following packages are remove candidates: %s'
        return True

    
    def runPostInstallScripts(self):
        ''' 
        scripts that are run in any case after the distupgrade finished 
        whether or not it was successfull
        '''
        for script in self.config.getlist('Distro', 'PostInstallScripts'):
            if not os.path.exists(script):
                logging.warning("PostInstallScript: '%s' not found" % script)
                continue
            
            logging.debug("Running PostInstallScript: '%s'" % script)
            
            try:
                os.chmod(script, 493)
                self._view.getTerminal().call([
                    script], hidden = True)
            continue
            except Exception:
                e = None
                logging.error('got error from PostInstallScript %s (%s)' % (script, e))
                continue
            

        

    
    def abort(self):
        ''' abort the upgrade, cleanup (as much as possible) '''
        if hasattr(self, 'sources'):
            self.sources.restoreBackup(self.sources_backup_ext)
        
        if hasattr(self, 'aptcdrom'):
            self.aptcdrom.restoreBackup(self.sources_backup_ext)
        
        self._view.updateStatus(_('Restoring original system state'))
        self._view.abort()
        self.openCache()
        sys.exit(1)

    
    def _checkDep(self, depstr):
        ''' check if a given depends can be satisfied '''
        for or_group in apt_pkg.ParseDepends(depstr):
            logging.debug("checking: '%s' " % or_group)
            for dep in or_group:
                depname = dep[0]
                ver = dep[1]
                oper = dep[2]
                if not self.cache.has_key(depname):
                    logging.error("_checkDep: '%s' not in cache" % depname)
                    return False
                
                inst = self.cache[depname]
                instver = inst.installedVersion
                if instver != None and apt_pkg.CheckDep(instver, oper, ver) == True:
                    return True
                    continue
            
        
        logging.error("depends '%s' is not satisfied" % depstr)
        return False

    
    def checkViewDepends(self):
        ''' check if depends are satisfied '''
        logging.debug('checkViewDepends()')
        res = True
        depends = self.config.getlist('View', 'Depends')
        depends.extend(self.config.getlist(self._view.__class__.__name__, 'Depends'))
        for dep in depends:
            logging.debug("depends: '%s'", dep)
            res &= self._checkDep(dep)
            if not res:
                self._view.error(_('Required depends is not installed'), _("The required dependency '%s' is not installed. " % dep))
                sys.exit(1)
                continue
        
        return res

    
    def _verifyBackports(self):
        backportslist = self.config.getlist('PreRequists', 'Packages')
        i = 0
        noCache = apt_pkg.Config.Find('Acquire::http::No-Cache', 'false')
        maxRetries = self.config.getint('Network', 'MaxRetries')
        while i < maxRetries:
            self.doUpdate(showErrors = False)
            self.openCache()
            for pkgname in backportslist:
                if not self.cache.has_key(pkgname):
                    logging.error("Can not find backport '%s'" % pkgname)
                    raise NoBackportsFoundException, pkgname
                    continue
            
            if self._allBackportsAuthenticated(backportslist):
                break
            
            logging.debug('setting a cache control header to turn off caching temporarily')
            apt_pkg.Config.Set('Acquire::http::No-Cache', 'true')
            i += 1
        if i == maxRetries:
            logging.error('pre-requists item is NOT trusted, giving up')
            return False
        
        apt_pkg.Config.Set('Acquire::http::No-Cache', noCache)
        return True

    
    def _allBackportsAuthenticated(self, backportslist):
        if apt_pkg.Config.FindB('APT::Get::AllowUnauthenticated', False) == True:
            logging.warning('skip authentication check because of APT::Get::AllowUnauthenticated==true')
            return True
        
        
        try:
            b = self.config.getboolean('Distro', 'AllowUnauthenticated')
            if b:
                return True
        except ConfigParser.NoOptionError:
            e = None

        for pkgname in backportslist:
            pkg = self.cache[pkgname]
            for cand in pkg.candidateOrigin:
                if cand.trusted:
                    break
                    continue
            else:
                return False
        
        return True

    
    def isMirror(self, uri):
        ''' check if uri is a known mirror '''
        for mirror in self.valid_mirrors:
            if is_mirror(mirror, uri):
                return True
                continue
        
        return False

    
    def _getPreReqMirrorLines(self, dumb = False):
        ''' get sources.list snippet lines for the current mirror '''
        lines = ''
        sources = SourcesList(matcherPath = '.')
        for entry in sources.list:
            if entry.invalid or entry.disabled:
                continue
            
            if entry.type == 'deb' and self.isMirror(entry.uri) and not entry.uri.startswith('http://security.ubuntu.com') and not entry.uri.startswith('http://archive.ubuntu.com'):
                new_line = 'deb %s %s-backports main/debian-installer\n' % (entry.uri, self.fromDist)
                if new_line not in lines:
                    lines += new_line
                
            
            if dumb and entry.type == 'deb' and 'main' in entry.comps:
                lines += 'deb %s %s-backports main/debian-installer\n' % (entry.uri, self.fromDist)
                continue
        
        return lines

    
    def _addPreRequistsSourcesList(self, template, out, dumb = False):
        ''' add prerequists based on template into the path outfile '''
        logging.debug("writing prerequists sources.list at: '%s' " % out)
        outfile = open(out, 'w')
        mirrorlines = self._getPreReqMirrorLines(dumb)
        for line in open(template):
            template = Template(line)
            outline = template.safe_substitute(mirror = mirrorlines)
            outfile.write(outline)
            logging.debug("adding '%s' prerequists" % outline)
        
        outfile.close()
        return True

    
    def getRequiredBackports(self):
        ''' download the backports specified in DistUpgrade.cfg '''
        logging.debug('getRequiredBackports()')
        res = True
        backportsdir = os.path.join(os.getcwd(), 'backports')
        if not os.path.exists(backportsdir):
            os.mkdir(backportsdir)
        
        backportslist = self.config.getlist('PreRequists', 'Packages')
        if self.aptcdrom and not (self.useNetwork):
            logging.debug('Searching for pre-requists on CDROM')
            p = os.path.join(self.aptcdrom.cdrompath, 'dists/stable/main/dist-upgrader/binary-%s/' % apt_pkg.Config.Find('APT::Architecture'))
            found_pkgs = set()
            for udeb in glob.glob(p + '*_*.udeb'):
                logging.debug("copying pre-req '%s' to '%s'" % (udeb, backportsdir))
                found_pkgs.add(os.path.basename(udeb).split('_')[0])
                shutil.copy(udeb, backportsdir)
            
            if not set(backportslist) == found_pkgs:
                logging.error("Expected backports: '%s' but got '%s'" % (set(backportslist), found_pkgs))
                return False
            
            return self.setupRequiredBackports(backportsdir)
        
        conf_option = 'SourcesList'
        if self.config.has_option('PreRequists', conf_option + '-%s' % self.arch):
            conf_option = conf_option + '-%s' % self.arch
        
        prereq_template = self.config.get('PreRequists', conf_option)
        if not os.path.exists(prereq_template):
            logging.error("sourceslist not found '%s'" % prereq_template)
            return False
        
        outpath = os.path.join(apt_pkg.Config.FindDir('Dir::Etc::sourceparts'), prereq_template)
        outfile = os.path.join(apt_pkg.Config.FindDir('Dir::Etc::sourceparts'), prereq_template)
        self._addPreRequistsSourcesList(prereq_template, outfile)
        
        try:
            self._verifyBackports()
        except NoBackportsFoundException:
            e = None
            self._addPreRequistsSourcesList(prereq_template, outfile, dumb = True)
            
            try:
                self._verifyBackports()
            except NoBackportsFoundException:
                e = None
                logging.warning("no backport for '%s' found" % e)

            return False

        cachedir = apt_pkg.Config.Find('Dir::Cache::archives')
        cwd = os.getcwd()
        if not os.path.exists(os.path.join(backportsdir, 'partial')):
            os.mkdir(os.path.join(backportsdir, 'partial'))
        
        os.chdir(backportsdir)
        apt_pkg.Config.Set('Dir::Cache::archives', backportsdir)
        for pkgname in backportslist:
            pkg = self.cache[pkgname]
            ver = self.cache._depcache.GetCandidateVer(pkg._pkg)
            if not ver:
                logging.error("No candidate for '%s'" % pkgname)
                os.unlink(outpath)
                return False
            
            if ver.FileList == None:
                logging.error("No ver.FileList for '%s'" % pkgname)
                os.unlink(outpath)
                return False
            
            logging.debug("marking '%s' for install" % pkgname)
            pkg.markInstall(autoFix = False)
        
        fetcher = apt_pkg.GetAcquire(self._view.getFetchProgress())
        pm = apt_pkg.GetPackageManager(self.cache._depcache)
        
        try:
            res = True
            self.cache._fetchArchives(fetcher, pm)
        except IOError:
            e = None
            logging.error("_fetchArchives returned '%s'" % e)
            res = False

        if res == False:
            logging.warning('_fetchArchives for backports returned False')
        
        os.unlink(outpath)
        apt_pkg.Config.Set('Dir::Cache::archives', cachedir)
        os.chdir(cwd)
        return self.setupRequiredBackports(backportsdir)

    
    def setupRequiredBackports(self, backportsdir):
        ''' setup the required backports in a evil way '''
        if not glob.glob(backportsdir + '/*.udeb'):
            logging.error('no backports found in setupRequiredBackports()')
            return False
        
        for deb in glob.glob(backportsdir + '/*.udeb'):
            logging.debug("extracting udeb '%s' " % deb)
            if os.system('dpkg-deb -x %s %s' % (deb, backportsdir)) != 0:
                return False
                continue
        
        os.environ['LD_LIBRARY_PATH'] = backportsdir + '/usr/lib'
        os.environ['PYTHONPATH'] = backportsdir + '/usr/lib/python%s.%s/site-packages/' % (sys.version_info[0], sys.version_info[1])
        os.environ['PATH'] = '%s:%s' % (backportsdir + '/usr/bin', os.environ['PATH'])
        logging.shutdown()
        shutil.copy('/var/log/dist-upgrade/main.log', '/var/log/dist-upgrade/main_pre_req.log')
        args = sys.argv + [
            '--have-prerequists']
        if self.useNetwork:
            args.append('--with-network')
        else:
            args.append('--without-network')
        if not S_IMODE(os.stat(sys.argv[0])[ST_MODE]) & S_IXUSR == S_IXUSR:
            os.chmod(sys.argv[0], 493)
        
        os.execve(sys.argv[0], args, os.environ)

    
    def preDoDistUpgrade(self):
        ''' this runs right before apt calls out to dpkg '''
        if os.path.exists('/usr/bin/killall'):
            subprocess.call([
                'killall',
                '-q',
                'update-notifier'])
        
        if self.fromDist == 'dapper' and 'DISPLAY' in os.environ and 'SUDO_USER' in os.environ:
            out = subprocess.Popen([
                'sudo',
                '-u',
                os.environ['SUDO_USER'],
                './theme-switch-helper.py',
                '-g'], stdout = subprocess.PIPE).communicate()[0]
            if 'Crux' in out:
                subprocess.call([
                    'sudo',
                    '-u',
                    os.environ['SUDO_USER'],
                    './theme-switch-helper.py',
                    '--defaults'])
            
        
        return True

    
    def fullUpgrade(self):
        self._view.updateStatus(_('Checking package manager'))
        self._view.setStep(DistUpgradeView.STEP_PREPARE)
        if not self.prepare():
            logging.error('self.prepared() failed')
            self._view.error(_('Preparing the upgrade failed'), _("Preparing the system for the upgrade failed. Please report this as a bug against the 'update-manager' package and include the files in /var/log/dist-upgrade/ in the bugreport."))
            sys.exit(1)
        
        if self.config.has_section('PreRequists') and self.options and self.options.havePrerequists == False:
            logging.debug('need backports')
            if not self.getRequiredBackports():
                self._view.error(_('Getting upgrade prerequisites failed'), _("The system was unable to get the prerequisites for the upgrade. The upgrade will abort now and restore the original system state.\n\nPlease report this as a bug against the 'update-manager' package and include the files in /var/log/dist-upgrade/ in the bugreport."))
                self.abort()
            
        
        self.doUpdate(showErrors = False, forceRetries = 1)
        self.openCache()
        if not self.doPostInitialUpdate():
            self.abort()
        
        self._view.setStep(DistUpgradeView.STEP_MODIFY_SOURCES)
        self._view.updateStatus(_('Updating repository information'))
        if not self.updateSourcesList():
            self.abort()
        
        if self.aptcdrom and not self.aptcdrom.add(self.sources_backup_ext):
            sys.exit(1)
        
        if not self.doUpdate():
            self.abort()
        
        self._view.updateStatus(_('Checking package manager'))
        self.openCache()
        for pkg in self.config.getlist('Distro', 'BaseMetaPkgs'):
            if not self.cache.has_key(pkg):
                logging.error("No '%s' after sources.list rewrite+update")
                self._view.error(_('Invalid package information'), _("After your package information was updated the essential package '%s' can not be found anymore.\nThis indicates a serious error, please report this bug against the 'update-manager' package and include the files in /var/log/dist-upgrade/ in the bugreport.") % pkg)
                self.abort()
                continue
        
        self._view.updateStatus(_('Calculating the changes'))
        if not self.askDistUpgrade():
            self.abort()
        
        self._view.setStep(DistUpgradeView.STEP_FETCH)
        self._view.updateStatus(_('Fetching'))
        if not self.doDistUpgradeFetching():
            self.abort()
        
        self.preDoDistUpgrade()
        self._view.setStep(DistUpgradeView.STEP_INSTALL)
        self._view.updateStatus(_('Upgrading'))
        if not self.doDistUpgrade():
            self.runPostInstallScripts()
            self._view.information(_('Upgrade complete'), _('The upgrade is completed but there were errors during the ugprade process.'))
            sys.exit(1)
        
        self._view.setStep(DistUpgradeView.STEP_CLEANUP)
        self._view.updateStatus(_('Searching for obsolete software'))
        self.doPostUpgrade()
        self._view.setStep(DistUpgradeView.STEP_REBOOT)
        self._view.updateStatus(_('System upgrade is complete.'))
        if self._view.confirmRestart():
            p = subprocess.Popen('/sbin/reboot')
            sys.exit(0)
        

    
    def run(self):
        self._view.processEvents()
        self.fullUpgrade()

    
    def doPartialUpgrade(self):
        ''' partial upgrade mode, useful for repairing '''
        STEP_PREPARE = STEP_PREPARE
        STEP_MODIFY_SOURCES = STEP_MODIFY_SOURCES
        STEP_FETCH = STEP_FETCH
        STEP_INSTALL = STEP_INSTALL
        STEP_CLEANUP = STEP_CLEANUP
        STEP_REBOOT = STEP_REBOOT
        import DistUpgrade.DistUpgradeView
        self._view.setStep(STEP_PREPARE)
        self._view.hideStep(STEP_MODIFY_SOURCES)
        self._view.hideStep(STEP_REBOOT)
        self._partialUpgrade = True
        self.prepare()
        if not self.doPostInitialUpdate():
            return False
        
        if not self.askDistUpgrade():
            return False
        
        self._view.setStep(STEP_FETCH)
        self._view.updateStatus(_('Fetching'))
        if not self.doDistUpgradeFetching():
            return False
        
        self._view.setStep(STEP_INSTALL)
        self._view.updateStatus(_('Upgrading'))
        if not self.doDistUpgrade():
            self._view.information(_('Upgrade complete'), _('The upgrade is completed but there were errors during the ugprade process.'))
            return False
        
        self._view.setStep(STEP_CLEANUP)
        if not self.doPostUpgrade():
            self._view.information(_('Upgrade complete'), _('The upgrade is completed but there were errors during the ugprade process.'))
            return False
        
        self._view.information(_('Upgrade complete'), _('The partial upgrade was completed.'))
        return True


if __name__ == '__main__':
    from DistUpgradeView import DistUpgradeView
    from DistUpgradeViewText import DistUpgradeViewText
    from DistUpgradeCache import MyCache
    logging.basicConfig(level = logging.DEBUG)
    v = DistUpgradeViewText()
    dc = DistUpgradeController(v)
    dc._disableAptCronJob()
    dc._enableAptCronJob()

