# Source Generated with Decompyle++
# File: in.pyc (Python 2.5)

'''Maintain a cache of icons.

If an icon is used more than once by a GUI then ensure that only one copy is
created.
'''

class IconCache(object):
    '''A cache of icons.'''
    
    def __init__(self, object_factory, qtgui_module):
        '''Initialise the cache.'''
        self._object_factory = object_factory
        self._qtgui_module = qtgui_module
        self._cache = []

    
    def get_icon(self, iconset):
        '''Return an icon described by the given iconset tag.'''
        iset = _IconSet(iconset)
        
        try:
            idx = self._cache.index(iset)
        except ValueError:
            idx = -1

        if idx >= 0:
            iset = self._cache[idx]
        else:
            name = 'icon'
            idx = len(self._cache)
            if idx > 0:
                name += str(idx)
            
            icon = self._object_factory.createQObject('QIcon', name, (), is_attribute = False)
            iset.set_icon(icon, self._qtgui_module)
            self._cache.append(iset)
        return iset.icon



class _IconSet(object):
    '''An icon set, ie. the mode and state and the pixmap used for each.'''
    
    def __init__(self, iconset):
        '''Initialise the icon set from an XML tag.'''
        self._fallback = iconset.text.replace('\\', '\\\\')
        self._use_fallback = True
        self._roles = { }
        for i in iconset:
            self._roles[i.tag] = i.text.replace('\\', '\\\\')
            self._use_fallback = False
        
        self.icon = None

    
    def set_icon(self, icon, qtgui_module):
        '''Save the icon and set its attributes.'''
        self.icon = icon

    
    def __eq__(self, other):
        '''Compare two icon sets for equality.'''
        if not isinstance(other, type(self)):
            return NotImplemented
        
        if self._use_fallback:
            if other._use_fallback:
                return self._fallback == other._fallback
            
            return False
        
        if other._use_fallback:
            return False
        
        return self._roles == other._roles


