# Source Generated with Decompyle++
# File: in.pyc (Python 2.5)

import sys
from sys import stdout, stderr
import copy

class IdentifierException(Exception):
    '''
    Raise if no identifier can be found
    '''
    pass


class OptionException(Exception):
    '''
    Raise when an option is not available.
    '''
    pass


class SectionException(Exception):
    '''
    Raise when a section is not available.
    '''
    pass


class ParseException(Exception):
    '''
    Raise when a postion is not available.
    '''
    pass


class Parser:
    '''
    Only low-level methods here.
    
    See the xutils.XUtils subclass for higher-level methods.
    '''
    
    def __init__(self, source = None):
        '''
        source = can be an object or a file. If set to None (default)
                 Parser will start from scratch with an empty
                 configuration.
        
        Public:
        
        self.comments = stores the commented lines located outside of the
                        sections in the xorg.conf
        
        self.globaldict = a global dictionary containing all the sections
                          and options. For further information on 
                          self.globaldict, have a look at self.__process()
                          and at getValue().
        
        self.requireid = a list of the sections which require to have an
                         "Identifier" set in the xorg.conf (e.g. Device
                         sections).
        
        self.identifiers = a dictionary of the sections which require identi
                           fiers.
        
        self.sections = a tuple containing the names of all
                        the sections which self.__process() will look for
                        in the xorg.conf. Sections with other names
                        will be ignored by self.__process().
        
        self.references = a list containing the names of all the possible
                          references.
        
        '''
        self.subsection = 'SubSection'
        self.source = source
        self.sections = ('InputDevice', 'Device', 'Module', 'Monitor', 'Screen', 'ServerLayout', 'ServerFlags', 'Extensions', 'Files', 'DRI', 'VideoAdaptor', 'Vendor', 'Modes', 'SubSection')
        self.requireid = [
            'InputDevice',
            'Device',
            'Monitor',
            'Screen',
            'ServerLayout']
        self.references = [
            'Device',
            'InputDevice',
            'Monitor',
            'Screen']
        self.identifiers = { }.fromkeys(self.requireid)
        self.comments = []
        self.globaldict = { }.fromkeys(self.sections, 0)
        for elem in self.globaldict:
            self.globaldict[elem] = { }
        
        self._Parser__process()

    
    def __process(self):
        """
        Perform a sanity check of the file and fill self.globaldict with
        all the sections and subsections in the xorg.conf
        
        
        empty = is the file empty? If yes, then don't check if:
            * the last section is not complete
            * there are duplicates
        
        hasSection:
            * == True: a section is already open
            * == False: a section is closed and/or a new section can be opened
            
        hasSubSection:
            * == True: a subsection is already open
            * == False: a section is closed and/or a new section can be opened
            
        sectionFlag: 
            * == '':a section is closed and/or a new section can be opened
            * == the name the current section
        
        sectionTags = counter of the number of Section and EndSection strings
        subSectionTags = counter of the number of SubSection and EndSubSection
                         strings
        
        linesList = the list of the lines in the source object.
        
        globaliters = counts how many times each kind of section
                             (sectionFlag) is found in the xorg.conf
        """
        file = self.source
        if file == None:
            linesList = []
        elif not hasattr(file, 'write'):
            myfile = open(file, 'r')
            linesList = myfile.readlines()
            myfile.close()
        else:
            linesList = file.readlines()
        globaliters = { }.fromkeys(self.sections, 0)
        empty = True
        hasSection = False
        hasSubSection = False
        sectionFlag = ''
        sectionTags = 0
        subSectionTags = 0
        it = 0
        for line in linesList:
            if line.strip().startswith('#'):
                if hasSection == False:
                    self.comments.append(line)
                
            elif line.lower().strip().startswith('section'):
                testLineFound = False
                for sect in self.sections:
                    if line.lower().find('"' + sect.lower() + '"') != -1:
                        testLineFound = True
                        section = sect
                        break
                        continue
                
                if not testLineFound:
                    error = 'The name in the following line is invalid for a section:\n%s' % line
                    raise ParseException(error)
                elif hasSection == False:
                    sectionTags += 1
                    sectionFlag = section
                    empty = False
                    hasSection = True
                else:
                    error = 'Sections cannot be nested in other sections.'
                    raise ParseException(error)
            elif line.lower().strip().startswith('endsection') == True:
                sectionTags += 1
                if hasSection == True and hasSubSection == False:
                    globaliters[sectionFlag] += 1
                    sectionFlag = ''
                    hasSection = False
                else:
                    error = 'An EndSection is in the wrong place.'
                    raise ParseException(error)
            elif line.lower().strip().startswith('subsection') == True:
                subSectionTags += 1
                if hasSection == True and hasSubSection == False:
                    hasSubSection = True
                    subSectionId = line[line.find('"') + 1:line.rfind('"')].strip()
                    self.globaldict.setdefault(self.subsection, { })
                    curlength = globaliters[self.subsection]
                    self.globaldict[self.subsection][curlength] = { }
                    self.globaldict[self.subsection][curlength]['section'] = sectionFlag
                    
                    try:
                        self.globaldict[self.subsection][curlength]['position'] = globaliters[sectionFlag]
                    except KeyError:
                        error = 'SubSections can be nested only in well formed sections.'
                        raise ParseException(error)

                    self.globaldict[self.subsection][curlength]['identifier'] = subSectionId
                    self.globaldict[self.subsection][curlength]['options'] = []
                else:
                    error = 'SubSections can be nested only in well formed sections.'
                    raise ParseException(error)
            elif line.lower().strip().startswith('endsubsection') == True:
                subSectionTags += 1
                if hasSubSection == True:
                    hasSubSection = False
                    globaliters[self.subsection] += 1
                else:
                    error = 'SubSections can be closed only after being previously opened.'
                    raise ParseException(error)
            elif sectionFlag != '':
                if line.strip() != '':
                    if hasSubSection == True:
                        self.globaldict[self.subsection][curlength]['options'].append('\t' + line.strip() + '\n')
                    else:
                        self.globaldict.setdefault(sectionFlag, { })
                        curlength = globaliters[sectionFlag]
                        self.globaldict[sectionFlag].setdefault(curlength, []).append('\t' + line.strip() + '\n')
                
            
            it += 1
        
        if not empty:
            if sectionTags % 2 != 0 or subSectionTags % 2 != 0:
                error = 'The last section is incomplete.'
                raise ParseException(error)
            
            self.getIds()
            self._Parser__complianceRules()
        else:
            self.getIds()

    
    def __complianceRules(self):
        '''
        This method contains the several checks which can guarantee
        compliance with the syntax rules of the xorg.conf
        '''
        if len(self.getDuplicateSections()) > 0:
            error = 'There cannot be Duplicate Sections:\n%s' % str(self.getDuplicateSections())
            raise ParseException(error)
        
        self.validateOptions()
        broken = self.getBrokenReferences()
        it = 0
        for section in broken:
            it += len(broken[section])
        
        if it > 0:
            error = 'There cannot be Broken References:\n%s' % str(broken)
            raise ParseException(error)
        
        for section in self.requireid:
            if len(self.globaldict[section]) != len(self.identifiers[section]):
                error = 'Not all the sections which require an identifier have an identifier.'
                raise ParseException(error)
                continue
        
        if len(self.globaldict['ServerLayout']) > 0:
            for section in self.globaldict['ServerLayout']:
                screenReferences = self.getReferences('ServerLayout', section, reflist = [
                    'Screen'])
                if len(screenReferences['Screen']) == 0:
                    error = 'The ServerLayout section must have at least 1 reference to a "Screen" section.'
                    raise ParseException(error)
                    continue
            
        
        defaultLayout = self.getDefaultServerLayout()
        if len(defaultLayout) > 0:
            if len(defaultLayout) > 1:
                error = 'No more than one default ServerLayout can be specified in the ServerFlags section.'
                raise ParseException(error)
            
            if not self.isSection('ServerLayout', position = defaultLayout[0]):
                error = 'The default ServerLayout does not exist'
                raise ParseException(error)
            
        

    
    def getIds(self):
        """
        Fill self.identifiers so that it has the section types as keys and a
        list of tuples as values. The tuples contain the identifier and the
        position of each section.
        
        NOTE: this method is called automatically in xutils when a section is
        created but in xorgparser it is necessary to call it manually after
        creating a new section which requires an identifier
        
        Here's a basic scheme:
        
        self.identifiers = {section_type1: [
                                        (identifier1, position1),
                                        (identifier2, position2)
                                      ], etc.
                           }
        
        Concrete example:
        
        self.identifiers = {'Device': [
                                        ('Configured Video Device', 0),
                                        ('Another Video Device', 1)
                                      ],
                            'Screen': [
                                        ('Configured Screen Device', 0),
                                        ('Another Screen Device', 1)
                                      ],
                           }
        
        """
        for sect in self.requireid:
            self.identifiers[sect] = []
            it = 0
            for elem in self.globaldict[sect]:
                
                try:
                    identifier = self.getValue(sect, 'Identifier', it)
                except (OptionException, SectionException):
                    error = 'No Identifier for section %s, position %d, can be found.' % (sect, elem)
                    raise ParseException(error)

                
                try:
                    identifier.append('')
                    identifier = identifier[0]
                except AttributeError:
                    pass

                self.identifiers[sect].append((identifier, it))
                it += 1
            
        

    
    def validateOptions(self):
        '''
        One word entries are not acceptable as either options or references.
        If one is found, ParseException will be raised.
        '''
        sectionsWhitelist = [
            'Files']
        optionsWhitelist = [
            'endmode']
        for section in self.sections:
            if section not in sectionsWhitelist:
                for position in self.globaldict[section]:
                    if section == 'SubSection':
                        options = self.globaldict[section][position]['options']
                    else:
                        options = self.globaldict[section][position]
                    for option in options:
                        option = option.strip()
                        if option.find('#') != -1:
                            option = option[0:option.find('#')]
                        
                        error = 'The following option is invalid: %s' % option.strip()
                        optbits = self._Parser__cleanForDuplicates(option, includenull = True)
                        if len(optbits) == 1 and optbits[0].strip().lower() not in optionsWhitelist:
                            raise ParseException(error)
                        
                        if not optbits[0][0].isalpha():
                            raise ParseException(error)
                            continue
                    
                
        

    
    def getDuplicateOptions(self, section, position):
        '''
        See if there are duplicate options in a section (it is ok to have duplicated
        references) e.g. several Load options, or Screen, etc.
        '''
        blacklist = [
            'driver',
            'busid',
            'identifier']
        total = []
        duplicates = []
        if section == 'SubSection':
            options = self.globaldict[section][position]['options']
        else:
            options = self.globaldict[section][position]
        for option in options:
            option = option.strip()
            if option.find('#') != -1:
                option = option[0:option.find('#')]
            
            optbits = self._Parser__cleanForDuplicates(option)
            
            try:
                if optbits[0].lower() in blacklist:
                    total.append(optbits[0])
                elif optbits[0].lower() == 'option':
                    if len(optbits) > 1 and optbits[1] != None:
                        total.append(optbits[1])
                    
            continue
            except (AttributeError, IndexError):
                continue
            

        
        final = { }
        for option in total:
            if final.get(option) != None:
                duplicates.append(option)
                continue
            None<EXCEPTION MATCH>(AttributeError, IndexError)
            final[option] = option
        
        return duplicates

    
    def checkDuplicateOptions(self):
        '''
        Look for and return duplicate options in all sections
        '''
        duplicates = { }
        for section in self.globaldict:
            for elem in self.globaldict[section]:
                duplopt = self.getDuplicateOptions(section, elem)
                if len(duplopt) > 0:
                    duplicates.setdefault(section, { }).setdefault(elem, duplopt)
                    continue
            
        
        return duplicates

    
    def __cleanForDuplicates(self, option, includenull = None):
        '''
        clean the option and return all its components in a list
        
        includenull - is used only by validateOptions() and makes
        sure that options with a null value assigned in quotation
        marks are not considered as one-word options
        '''
        optbits = []
        optbit = ''
        it = 0
        quotation = 0
        optcount = option.count('"')
        if optcount > 0:
            for i in option:
                if not i.isspace():
                    if i == '"':
                        quotation += 1
                    else:
                        optbit += i
                elif quotation % 2 != 0:
                    optbit += i
                elif len(optbit) > 0:
                    optbits.append(optbit)
                    optbit = ''
                
                if it == len(option) - 1:
                    if optbit != '':
                        optbits.append(optbit)
                    
                
                it += 1
            
        else:
            for i in option:
                if not i.isspace():
                    optbit += i
                elif len(optbit) > 0:
                    optbits.append(optbit)
                    optbit = ''
                
                if it == len(option) - 1:
                    if optbit != '':
                        optbits.append(optbit)
                    elif includenull:
                        optbit = ''
                        optbits.append(optbit)
                    
                
                it += 1
            
        if includenull and len(optbits) != optcount / 2 + 1:
            optbit = ''
            optbits.append(optbit)
        
        return optbits

    
    def getDuplicateSections(self):
        '''
        Return a dictionary with the duplicate sections i.e. sections
        of the same kind, with the same identifier
        '''
        duplicates = { }
        for section in self.identifiers:
            temp = []
            for sect in self.identifiers[section]:
                temp.append(sect[0])
            
            for elem in temp:
                if temp.count(elem) > 1:
                    duplicates.setdefault(section, { }).setdefault(elem, temp.count(elem))
                    continue
            
        
        return duplicates

    
    def addOption(self, section, option, value, optiontype = None, position = None, reference = None, prefix = '"'):
        '''
        Add an option to a section
        
        section= the section which will have the option added
        option= the option to add
        value= the value which will be assigned to the option
        position= e.g. 0 (i.e. the first element in the list of Screen
                      sections)
        optiontype= if set to "Option" it will cause the option to look like
                    the following:
                    Option "NameOfTheOption" "Value"
                    
                    Otherwise it will look like the following:
                    NameOfTheOption "Value"
        position= e.g. 0 (i.e. the first element in the list of Screen
                      sections)
        reference= used only in a particular case of reference (see addReference)
        
        prefix= usually quotation marks are used for the values (e.g. "True")
                however sometimes they don\'t have to be used
                (e.g. DefaultDepth 24) and prefix should be set to \'\' instead of
                \'"\'
        '''
        refSections = [
            'device']
        if position != None:
            if self.globaldict[section].get(position) == None:
                raise SectionException
            
            if reference:
                self.removeOption(section, option, value = value, position = position)
            else:
                self.removeOption(section, option, position = position)
        else:
            self.removeOption(section, option)
        if optiontype == None:
            if reference == None:
                toadd = '\t' + option + '\t' + prefix + str(value) + prefix + '\n'
            elif section.strip().lower() not in refSections:
                toadd = '\t' + option + '\t' + prefix + str(value) + prefix + '\n'
            else:
                toadd = '\t' + option + '\t' + str(value) + '\n'
        else:
            toadd = '\t' + optiontype + '\t' + prefix + option + prefix + '\t' + prefix + str(value) + prefix + '\n'
        if len(self.globaldict[section]) == 0:
            self.globaldict[section] = { }
            self.globaldict[section][0] = []
            if section in self.requireid:
                identifier = '\tIdentifier\t"Default ' + section + '"\n'
                self.globaldict[section][0].append(identifier)
            
        
        if position == None:
            for elem in self.globaldict[section]:
                self.globaldict[section][elem].append(toadd)
            
        else:
            self.globaldict[section][position].append(toadd)

    
    def removeOption(self, section, option, value = None, position = None, reference = None):
        '''
        Remove an option from a section
        
        section= the section which will have the option removed
        option= the option to remove
        value= if you want to remove an option only if it has a certain value
        position= e.g. 0 (i.e. the first element in the list of Screen
                      sections)
        '''
        toremove = { }
        if len(self.globaldict[section]) != 0:
            if position == None:
                for elem in self.globaldict[section]:
                    it = 0
                    for line in self.globaldict[section][elem]:
                        if value != None:
                            if line.lower().find(option.lower()) != -1 and line.lower().find(value.lower()) != -1:
                                toremove.setdefault(elem, []).append(it)
                            
                        elif line.lower().find(option.lower()) != -1:
                            toremove.setdefault(elem, []).append(it)
                        
                        it += 1
                    
                
            elif self.globaldict[section].get(position) == None:
                return None
            else:
                it = 0
                for line in self.globaldict[section][position]:
                    if value != None:
                        if line.lower().find(option.lower()) != -1 and line.lower().find(value.lower()) != -1:
                            toremove.setdefault(position, []).append(it)
                        
                    elif line.lower().find(option.lower()) != -1:
                        toremove.setdefault(position, []).append(it)
                    
                    it += 1
                
            for part in toremove:
                modded = 0
                for line in toremove[part]:
                    realpos = line - modded
                    del self.globaldict[section][part][realpos]
                    modded += 1
                
            
        

    
    def makeSection(self, section, identifier = None):
        '''
        Create a new section and return the position of the section in the list
        of sections of the same type (e.g. "Screen") so as to make it available
        in case the user wants to add some options to it
        
        The identifier and the position of the new section is added to 
        self.identifiers[section]
        
        section= the section to create
        identifier= the identifier of a section (if the section requires an
                    identifier)
        '''
        position = len(self.globaldict[section])
        if section in self.requireid:
            if identifier != None:
                option = 'Identifier'
                create = True
                for sub in self.globaldict[section]:
                    if self.getValue(section, option, sub):
                        
                        try:
                            if self.getValue(section, option, sub).strip().lower() == identifier.strip().lower():
                                create = False
                                break
                        except AttributeError:
                            for elem in self.getValue(section, option, sub):
                                if elem.strip().lower() == identifier.strip().lower():
                                    create = False
                                    break
                                    continue
                            
                        

                    None<EXCEPTION MATCH>AttributeError
                
                if create:
                    self.globaldict[section][position] = []
                    self.addOption(section, option, value = identifier, position = position)
                    self.identifiers[section].append((identifier, position))
                
            else:
                raise IdentifierException('%s Section requires an identifier' % section)
        else:
            self.globaldict[section][position] = []
        return position

    
    def addReference(self, section, reference, identifier, position = None):
        '''
        Add a reference to a section from another section
        For example:
        to put a reference to the Screen section named "Default Screen"
        in the ServerLayout section you should do:
        
        section=\'ServerLayout\'
        reference=\'Screen\'
        identifier=\'Default Screen\'
        position=0 #the first ServerLayout section
        
        NOTE: if position is set to None it will add such reference to any
        instance of the section (e.g. to any ServerLayout section)
        '''
        self.addOption(section, reference, value = identifier, position = position, reference = True)

    
    def removeReference(self, section, reference, identifier, position = None):
        '''
        Remove a reference to a section from another section
        For example:
        to remove a reference to Screen "Default Screen" from the
        ServerLayout section you should do:
        
        section=\'ServerLayout\'
        reference=\'Screen\'
        identifier=\'Default Screen\'
        position=0 #the first ServerLayout section
                
        NOTE: if position is set to None it will remove such reference from any
        instance of the section (e.g. from any ServerLayout section)
        '''
        self.removeOption(section, reference, value = identifier, position = position, reference = True)

    
    def getReferences(self, section, position, reflist = None):
        '''
        Get the references to other sections which are located in a section.
        
        section= the section (e.g. "Screen")
        position= e.g. 0 stands for the 1st Screen section
        reflist= a list of references which this function should look for.
                 The default list of references is self.requireid but this
                 list can be overridden by the reflist argument so that, for
                 example, if reflist is set to [\'Device\'], this function will
                 look for references to other devices only (references to, say,
                 screens, will be ignored).
        '''
        if reflist == None:
            options = self.requireid
        else:
            reflist.append('')
            del reflist[-1]
            options = reflist
        references = { }.fromkeys(options)
        for option in options:
            references[option] = []
            referenceDict = { }
            
            try:
                ref = self.getValue(section, option, position, reference = True)
            except OptionException:
                ref = []

            if ref:
                
                try:
                    ref.append('')
                    del ref[-1]
                    for elem in ref:
                        
                        try:
                            elem.append('')
                            del elem[-1]
                            for extref in elem:
                                if elem:
                                    referenceDict.setdefault(extref)
                                    continue
                        continue
                        except AttributeError:
                            if elem:
                                referenceDict.setdefault(elem)
                            
                            elem
                        

                except AttributeError:
                    if ref:
                        referenceDict.setdefault(ref)
                    
                except:
                    ref

                for reference in referenceDict.keys():
                    references[option].append(reference)
                
        
        return references

    
    def makeSubSection(self, section, identifier, position = None):
        '''
        section= the section to which the subsection will belong
        identifier= the name of the subsection
        position= the position of the section in the dictionary with the
             sections (e.g. the 1st "Screen" section would be 0). If set to
             None it will create a new subsection in all the instances of
             the said section (e.g. in all the "Screen" sections)
        '''
        curlength = len(self.globaldict[self.subsection])
        if position == None:
            for elem in self.globaldict[section]:
                create = True
                for sub in self.globaldict[self.subsection]:
                    if self.globaldict[self.subsection][sub].get('section') == section and self.globaldict[self.subsection][sub].get('identifier') == identifier and self.globaldict[self.subsection][sub].get('position') == elem:
                        create = False
                        continue
                
                if create:
                    self.globaldict[self.subsection][curlength] = { }
                    self.globaldict[self.subsection][curlength]['section'] = section
                    self.globaldict[self.subsection][curlength]['identifier'] = identifier
                    self.globaldict[self.subsection][curlength]['options'] = []
                    self.globaldict[self.subsection][curlength]['position'] = elem
                    curlength += 1
                    continue
            
        else:
            create = True
            for sub in self.globaldict[self.subsection]:
                if self.globaldict[self.subsection][sub].get('section') == section and self.globaldict[self.subsection][sub].get('identifier') == identifier and self.globaldict[self.subsection][sub].get('position') == position:
                    create = False
                    continue
            
            if create:
                self.globaldict[self.subsection][curlength] = { }
                self.globaldict[self.subsection][curlength]['section'] = section
                self.globaldict[self.subsection][curlength]['identifier'] = identifier
                self.globaldict[self.subsection][curlength]['options'] = []
                self.globaldict[self.subsection][curlength]['position'] = position
            

    
    def removeSubSection(self, section, identifier, position = None):
        '''
        section= the section to which the subsection belongs
        identifier= the name of the subsection
        position= the position of the section in the dictionary with the
             sections (e.g. the 1st "Screen" section would be 0). If set to
             None it will remove a subsection from all the instances of
             the said section (e.g. in all the "Screen" sections)
        '''
        curlength = len(self.globaldict[self.subsection])
        toremove = []
        if position == None:
            for elem in self.globaldict[self.subsection]:
                if self.globaldict[self.subsection][elem].get('section') == section and self.globaldict[self.subsection][elem].get('identifier') == identifier:
                    toremove.append(elem)
                    continue
            
        else:
            for elem in self.globaldict[self.subsection]:
                if self.globaldict[self.subsection][elem].get('section') == section and self.globaldict[self.subsection][elem].get('identifier') == identifier and self.globaldict[self.subsection][elem].get('position') == position:
                    toremove.append(elem)
                    continue
            
        for item in toremove:
            del self.globaldict[self.subsection][item]
        

    
    def addSubOption(self, section, identifier, option, value, optiontype = None, position = None):
        '''
        Adds an option to one or more subsections
        
        section= the section which contains the subsection
        identifier= the identifier of the SubSection (e.g. Display)
        option= the option to add
        value= the value which will be assigned to the option
        optiontype= if set to "Option" it will cause the option to look like
                    the following:
                    Option "NameOfTheOption" "Value"
                    
                    Otherwise it will look like the following:
                    NameOfTheOption "Value"
        position= e.g. 0 (i.e. the option will be added to a subsection which
                  is located in the first element in the list of Screen
                  sections)
        '''
        prefix = '"'
        donotcreate = []
        tomodify = []
        if position == None:
            self.removeSubOption(section, identifier, option)
        else:
            self.removeSubOption(section, identifier, option, position = position)
        if optiontype == None:
            toadd = '\t' + option + '\t' + str(value) + '\n'
        else:
            toadd = '\t' + optiontype + '\t' + prefix + option + prefix + '\t' + prefix + str(value) + prefix + '\n'
        curlength = len(self.globaldict[self.subsection])
        if curlength == 0:
            self.globaldict[self.subsection][0] = {
                'section': section,
                'identifier': identifier,
                'options': [] }
        
        if position == None:
            cursectlength = len(self.globaldict[section])
            it = 0
            while it < cursectlength:
                for elem in self.globaldict[self.subsection]:
                    if self.globaldict[self.subsection][elem].get('position') == it and self.globaldict[self.subsection][elem].get('section') == section and self.globaldict[self.subsection][elem].get('identifier') == identifier:
                        donotcreate.append(it)
                        continue
                
                it += 1
            for i in range(cursectlength + 1):
                if i not in donotcreate:
                    self.makeSubSection(section, identifier, position = i)
                    continue
            
            for elem in self.globaldict[self.subsection]:
                if self.globaldict[self.subsection][elem].get('identifier') == identifier and self.globaldict[self.subsection][elem].get('section') == section:
                    tomodify.append(elem)
                    continue
            
        else:
            for elem in self.globaldict[self.subsection]:
                if self.globaldict[self.subsection][elem].get('position') == position and self.globaldict[self.subsection][elem].get('identifier') == identifier:
                    tomodify.append(elem)
                    continue
            
            if len(tomodify) == 0:
                curlength = len(self.globaldict[self.subsection])
                self.globaldict[self.subsection][len(self.globaldict[self.subsection])] = {
                    'section': section,
                    'identifier': identifier,
                    'options': [],
                    'position': position }
                tomodify.append(curlength)
            
        for elem in tomodify:
            self.globaldict[self.subsection][elem]['options'].append(toadd)
        

    
    def removeSubOption(self, section, identifier, option, position = None):
        '''
        Remove an option from a SubSection.
        
        See addSubOption() for an explanation on the arguments.
        '''
        toremove = { }
        if len(self.globaldict[section]) != 0:
            if len(self.globaldict[self.subsection]) != 0:
                for elem in self.globaldict[self.subsection]:
                    if position == None:
                        if self.globaldict[self.subsection][elem].get('section') == section and self.globaldict[self.subsection][elem].get('identifier') == identifier:
                            it = 0
                            for opt in self.globaldict[self.subsection][elem]['options']:
                                if opt.strip().lower().find(option.strip().lower()) != -1:
                                    toremove.setdefault(elem, []).append(it)
                                
                                it += 1
                            
                        
                    self.globaldict[self.subsection][elem].get('identifier') == identifier
                    if self.globaldict[self.subsection][elem].get('section') == section and self.globaldict[self.subsection][elem].get('identifier') == identifier and self.globaldict[self.subsection][elem].get('position') == position:
                        it = 0
                        for opt in self.globaldict[self.subsection][elem]['options']:
                            if opt.strip().lower().find(option.strip().lower()) != -1:
                                toremove.setdefault(elem, []).append(it)
                            
                            it += 1
                        
                
                for elem in toremove:
                    modded = 0
                    for part in toremove[elem]:
                        realpos = part - modded
                        del self.globaldict[self.subsection][elem]['options'][realpos]
                        modded += 1
                    
                
            
        

    
    def getIdentifier(self, section, position):
        '''
        Get the identifier of a specific section from its position
        '''
        errorMsg = 'No identifier can be found for section "%s" No %d' % (section, position)
        
        try:
            for sect in self.identifiers[section]:
                if sect[1] == position:
                    return sect[0]
                    continue
        except KeyError:
            raise SectionException

        raise IdentifierException, errorMsg

    
    def __cleanOption(self, option, optname, reference = None, section = None):
        '''
        clean the option and return the value (i.e. the last item of the list
        which this method generates).
        
        If no value can be found, return False
        '''
        if reference:
            if 'option' in option.strip().lower():
                return False
            
            if not option.strip().lower().startswith(optname.strip().lower()):
                return False
            
        
        optbits = []
        optbit = ''
        it = 0
        quotation = 0
        optcount = option.count('"')
        if optcount > 0:
            for i in option:
                if optcount in (2, 4) and section == 'ServerLayout':
                    if not i.isspace():
                        if i == '"':
                            if quotation != 0 and quotation % 2 != 0:
                                if len(optbit) > 0:
                                    optbits.append(optbit)
                                    optbit = ''
                                
                            
                            quotation += 1
                        elif quotation % 2 != 0:
                            optbit += i
                        
                    elif quotation % 2 != 0:
                        optbit += i
                    
                elif not i.isspace():
                    if i == '"':
                        quotation += 1
                    else:
                        optbit += i
                elif quotation % 2 != 0:
                    optbit += i
                elif len(optbit) > 0:
                    optbits.append(optbit)
                    optbit = ''
                
                if it == len(option) - 1:
                    if optbit != '':
                        optbits.append(optbit)
                    
                
                it += 1
            
        else:
            for i in option:
                if not i.isspace():
                    optbit += i
                elif len(optbit) > 0:
                    optbits.append(optbit)
                    optbit = ''
                
                if it == len(option) - 1:
                    if optbit != '':
                        optbits.append(optbit)
                    
                
                it += 1
            
        optlen = len(optbits)
        if optlen > 1:
            referencesList = [ x.lower().strip() for x in self.references ]
            if section != 'ServerLayout' and quotation == 0 and optlen == 2 and optbits[0].lower().strip() in referencesList:
                
                try:
                    sect = ''
                    value = int(optbits[1].strip())
                    for item in self.requireid:
                        if optbits[0].lower().strip() == item.lower().strip():
                            sect = item
                            break
                            continue
                        []
                    
                    
                    try:
                        identifier = self.getIdentifier(sect, value)
                        return [
                            identifier]
                    except IdentifierException:
                        []
                        []
                        return False
                    except:
                        []

                except ValueError:
                    []
                    []
                    []
                except:
                    []<EXCEPTION MATCH>ValueError
                

            []
            if optcount != 4 and section != 'ServerLayout':
                status = False
                for elem in optbits:
                    if elem.lower() == optname.lower():
                        status = True
                        continue
                    []
                
                if status == False:
                    return False
                
            
            if optlen == 2 and optbits[0].lower().strip() == 'option':
                return 'True'
            
            sections = [ sect.strip().lower() for sect in self.sections ]
            if optcount == 4 and section == 'ServerLayout':
                serverDict = { }
                for elem in optbits:
                    serverDict.setdefault(elem)
                
                return serverDict.keys()
            elif optcount > 0 and optcount <= 4:
                return optbits[optlen - 1]
            elif optcount > 4:
                del optbits[0]
                return optbits
            elif optcount == 0:
                del optbits[0]
                return ' '.join(optbits)
            
        elif optcount in (2, 4) and section == 'ServerLayout':
            return optbits
        
        return False

    
    def getValue(self, section, option, position, identifier = None, sect = None, reference = None):
        '''
        Get the value which is assigned to an option.
        
        Return types:
          * string (if only one value is available)
            - usually in options
          * list (if more than one option is found)
            - having multiple references of the same type is allowed.
              for example it is not unusual to have 2 references to
              Screen sections in the ServerLayout section (in case of
              Xinerama)
            - if the options are actually options and not references
              then there are duplicate options, which should be detected
              in advance with xutils.XUtils.getDuplicateOptions()   
          * None (if no value can be found) - Not always true -> See below.
        
        NOTE: Use-case for returning None
            * When dealing with incomplete references. For example:
                  Screen "Configured Screen"
                is different from:
                  Screen
                  (which is an incomplete reference)
            * When dealing with incomplete options. For example:
                  Depth 24
                is different from:
                  Depth
                  (which is an incomplete option)
            * Exception:
                Some options (with the "Option" prefix) (not references)
                can be used with no value (explicitly) assigned and are
                considered as True by the Xserver. In such case getValue()
                will return "True". For example:
                  Option "AddARGBGLXVisuals" 
                is the same as:
                  Option "AddARGBGLXVisuals" "True"
        
        NOTE: Meaning of keys in Sections and SubSections:
            * When dealing with a Section:
                section= e.g. \'Screen\', \'Device\', etc.
                option= the option
                position= e.g. 0 (i.e. the first element in the list of Screen
                          sections)
                reference= used only by getReferences()
            
            * When dealing with a SubSection:
                section= \'SubSection\' (this is mandatory)
                option= the option
                position= e.g. 0 would mean that the subsection belongs to 
                          the 1st item of the list of, say, "Screen" sections.
                          (i.e. the first element in the list of Screen 
                          sections)
                          ["position" is a key of an item of the list of 
                          subsections see below]
                
                identifier= the name of the subsection e.g. \'Display\'
                sect = the \'section\' key of an item of the list of 
                       subsections e.g. the "Display" subsection can be 
                       found in the "Screen" section (\'sect\' is the latter)
            
        NOTE: Anatomy of Sections and SubSections:
            * Anatomy of subsections:
                self.globaldict[\'SubSection\'] =
                    {0: {\'section\': \'Screen\', \'identifier\': \'Display\', 
                    \'position\': 0, \'options\': [option1, option2, etc.], 
                    etc.}
                    In this case we refer to the \'Display\' subsection 
                    which is located in the first \'Screen\' section.
            
            * Anatomy of a section:
                self.globaldict[\'Screen\'] =
                    {0: [option1, option2, etc.], 1: [...], ...}
                0, 1, etc. is the position
            
        '''
        values = []

    
    def isSection(self, section, identifier = None, position = None):
        '''
        See if a section with a certain identifier exists
        
        NOTE: either identifier or position must be provided
        '''
        if identifier != None:
            
            try:
                self.getPosition(section, identifier)
                return True
            except IdentifierException:
                return False
            except:
                None<EXCEPTION MATCH>IdentifierException
            

        None<EXCEPTION MATCH>IdentifierException
        if position != None:
            return self.globaldict[section].get(position) != None
        else:
            errorMsg = 'Either identifier or position must be provided'
            raise Exception(errorMsg)

    
    def getPosition(self, section, identifier):
        '''
        Get the position of a specific section from its identifier
        '''
        errorMsg = 'No %s section named "%s" can be found' % (section, identifier)
        for sect in self.identifiers[section]:
            
            try:
                if sect[0].strip().lower() == identifier.strip().lower():
                    return sect[1]
            continue
            except AttributeError:
                continue
            

        
        raise IdentifierException, errorMsg

    
    def getBrokenReferences(self):
        """
        Look for broken references (i.e. references to sections which don't exist)
        and return a dictionary having the items of self.requireid as keys and
        a dictionary of the identifiers of the sections which are referred to by the
        broken references. For example:
        
        brokenReferences = {
                            'InputDevice': {'InputDevice 1': None, 'Another input device': None},
                            'Device': {...},
                            'Monitor' {...},
                            'Screen' {...},
                            'ServerLayout' {...}
                            }
        """
        brokenReferences = { }.fromkeys(self.requireid)
        referencesTree = { }
        for section in self.requireid:
            referencesTree[section] = { }
            brokenReferences[section] = { }
            for sect in self.globaldict[section]:
                referencesTree[section][sect] = self.getReferences(section, sect)
            
        
        for section in referencesTree:
            for elem in referencesTree[section]:
                for refsect in referencesTree[section][elem]:
                    if len(referencesTree[section][elem][refsect]) > 0:
                        for ref in referencesTree[section][elem][refsect]:
                            for sect in self.sections:
                                if sect.lower() == refsect.strip().lower():
                                    refsect = sect
                                    continue
                            
                            if not self.isSection(refsect, ref):
                                brokenReferences[refsect].setdefault(ref)
                                continue
                        
                
            
        
        return brokenReferences

    
    def getDefaultServerLayout(self):
        '''
        See if one or more ServerLayout sections are set as default and return their
        position in a list
        '''
        default = []
        serverFlags = self.globaldict['ServerFlags']
        it = 0
        for flag in serverFlags:
            
            try:
                defaultLayout = self.getValue('ServerFlags', 'DefaultServerLayout', it)
                if defaultLayout:
                    defIt = 0
                    for identifier in self.identifiers['ServerLayout']:
                        if identifier[0].lower().strip() == defaultLayout.lower().strip():
                            default.append(identifier[1])
                            defIt += 1
                            continue
                    
                    if defIt == 0:
                        error = 'The default ServerLayout does not exist.'
                        raise ParseException(error)
                    
            except OptionException:
                pass

            it += 1
        
        return default

    
    def __mergeSubSections(self):
        '''
        Put SubSections back into the sections to which they belong.
        '''
        self.tempdict = copy.deepcopy(self.globaldict)
        for sect in self.tempdict['SubSection']:
            section = self.tempdict['SubSection'][sect]['section']
            identifier = self.tempdict['SubSection'][sect]['identifier']
            position = self.tempdict['SubSection'][sect].get('position')
            options = self.tempdict['SubSection'][sect]['options']
            self.tempdict[section].setdefault(position, []).append('\tSubSection "' + identifier + '"' + '\n')
            if len(options) > 0:
                self.tempdict[section][position].append('\t' + '\t'.join(options) + '\tEndSubSection\n')
                continue
            self.tempdict[section][position].append('\t'.join(options) + '\tEndSubSection\n')
        
        
        try:
            del self.tempdict['SubSection']
        except KeyError:
            pass


    
    def writeFile(self, destination, test = None):
        """
        Write the changes to the destination file (e.g. /etc/X11/xorg.conf)
        or file object (e.g. sys.stdout).
        
        * Arguments:
          destination = the destination file or file object (mandatory)
          test = if set to True writeFile will append the result to the
                 destination file instead of overwriting it. It has no 
                 effect on file objects. Useful for testing.
        
        NOTE: global dict's state is not altered.
        """
        self._Parser__mergeSubSections()
        lines = []
        comments = ''.join(self.comments) + '\n'
        lines.append(comments)
        for section in self.tempdict:
            if len(self.tempdict[section]) > 0:
                for elem in self.tempdict[section]:
                    lines.append('Section "' + section + '"' + '\n')
                    lines.append(''.join(self.tempdict[section][elem]) + 'EndSection\n\n')
                
        
        del self.tempdict
        if not hasattr(destination, 'write'):
            if test:
                destination = open(destination, 'a')
            else:
                destination = open(destination, 'w')
            destination.write(''.join(lines))
            destination.close()
        else:
            destination.write(''.join(lines))


