# Source Generated with Decompyle++
# File: in.pyc (Python 2.5)

__version__ = ' $Id: colors.py 3001 2006-11-17 11:44:21Z jjlee $ '
import string
import math
from types import StringType, ListType, TupleType
from reportlab.lib.utils import fp_str
_SeqTypes = (ListType, TupleType)

class Color:
    '''This class is used to represent color.  Components red, green, blue
    are in the range 0 (dark) to 1 (full intensity).'''
    
    def __init__(self, red = 0, green = 0, blue = 0):
        '''Initialize with red, green, blue in range [0-1].'''
        self.red = red
        self.green = green
        self.blue = blue

    
    def __repr__(self):
        return 'Color(%s)' % string.replace(fp_str(self.red, self.green, self.blue), ' ', ',')

    
    def __hash__(self):
        return hash((self.red, self.green, self.blue))

    
    def __cmp__(self, other):
        
        try:
            dsum = ((4 * self.red - 4 * other.red) + 2 * self.green - 2 * other.green) + self.blue - other.blue
        except:
            return -1

        if dsum > 0:
            return 1
        
        if dsum < 0:
            return -1
        
        return 0

    
    def rgb(self):
        '''Returns a three-tuple of components'''
        return (self.red, self.green, self.blue)

    
    def bitmap_rgb(self):
        return tuple(map((lambda x: int(x * 255) & 255), self.rgb()))

    
    def hexval(self):
        return '0x%02x%02x%02x' % self.bitmap_rgb()



class CMYKColor(Color):
    '''This represents colors using the CMYK (cyan, magenta, yellow, black)
    model commonly used in professional printing.  This is implemented
    as a derived class so that renderers which only know about RGB "see it"
    as an RGB color through its \'red\',\'green\' and \'blue\' attributes, according
    to an approximate function.

    The RGB approximation is worked out when the object in constructed, so
    the color attributes should not be changed afterwards.

    Extra attributes may be attached to the class to support specific ink models,
    and renderers may look for these.'''
    
    def __init__(self, cyan = 0, magenta = 0, yellow = 0, black = 0, spotName = None, density = 1, knockout = None):
        '''
        Initialize with four colors in range [0-1]. the optional
        spotName, density & knockout may be of use to specific renderers.
        spotName is intended for use as an identifier to the renderer not client programs.
        density is used to modify the overall amount of ink.
        knockout is a renderer dependent option that determines whether the applied colour
        knocksout (removes) existing colour; None means use the global default.
        '''
        self.cyan = cyan
        self.magenta = magenta
        self.yellow = yellow
        self.black = black
        self.spotName = spotName
        self.density = max(min(density, 1), 0)
        self.knockout = knockout
        (self.red, self.green, self.blue) = cmyk2rgb((cyan, magenta, yellow, black))
        if density < 1:
            r = self.red
            g = self.green
            b = self.blue
            r = density * (r - 1) + 1
            g = density * (g - 1) + 1
            b = density * (b - 1) + 1
            self.red = r
            self.green = g
            self.blue = b
        

    
    def __repr__(self):
        if not self.spotName or ',spotName=' + repr(self.spotName):
            pass
        if not self.density != 1 or ',density=' + fp_str(self.density):
            pass
        if not self.knockout is not None or ',knockout=%d' % self.knockout:
            pass
        return 'CMYKColor(%s%s%s%s)' % (string.replace(fp_str(self.cyan, self.magenta, self.yellow, self.black), ' ', ','), '', '', '')

    
    def __hash__(self):
        return hash((self.cyan, self.magenta, self.yellow, self.black, self.density, self.spotName))

    
    def __cmp__(self, other):
        '''Partial ordering of colors according to a notion of distance.

        Comparing across the two color models is of limited use.'''
        if isinstance(other, CMYKColor):
            if not self.spotName:
                pass
            if not other.spotName:
                pass
            dsum = (((((self.cyan - other.cyan) * 2 + (self.magenta - other.magenta)) * 2 + (self.yellow - other.yellow)) * 2 + (self.black - other.black)) * 2 + (self.density - other.density)) * 2 + cmp('', '')
        else:
            
            try:
                dsum = ((self.red - other.red) * 2 + (self.green - other.green)) * 2 + (self.blue - other.blue)
            except:
                return -1

        if dsum >= 0:
            return dsum > 0
        else:
            return -1

    
    def cmyk(self):
        '''Returns a tuple of four color components - syntactic sugar'''
        return (self.cyan, self.magenta, self.yellow, self.black)

    
    def _density_str(self):
        return fp_str(self.density)



class PCMYKColor(CMYKColor):
    '''100 based CMYKColor with density and a spotName; just like Rimas uses'''
    
    def __init__(self, cyan, magenta, yellow, black, density = 100, spotName = None, knockout = None):
        CMYKColor.__init__(self, cyan / 100, magenta / 100, yellow / 100, black / 100, spotName, density / 100, knockout = knockout)

    
    def __repr__(self):
        if not self.spotName or ',spotName=' + repr(self.spotName):
            pass
        if not self.density != 1 or ',density=' + fp_str(self.density * 100):
            pass
        if not self.knockout is not None or ',knockout=%d' % self.knockout:
            pass
        return 'PCMYKColor(%s%s%s%s)' % (string.replace(fp_str(self.cyan * 100, self.magenta * 100, self.yellow * 100, self.black * 100), ' ', ','), '', '', '')



def cmyk2rgb(.0, density = 1):
    '''Convert from a CMYK color tuple to an RGB color tuple'''
    (c, m, y, k) = .0
    r = 1 - min(1, c + k)
    g = 1 - min(1, m + k)
    b = 1 - min(1, y + k)
    return (r, g, b)


def rgb2cmyk(r, g, b):
    '''one way to get cmyk from rgb'''
    c = 1 - r
    m = 1 - g
    y = 1 - b
    k = min(c, m, y)
    c = min(1, max(0, c - k))
    m = min(1, max(0, m - k))
    y = min(1, max(0, y - k))
    k = min(1, max(0, k))
    return (c, m, y, k)


def color2bw(colorRGB):
    '''Transform an RGB color to a black and white equivalent.'''
    col = colorRGB
    r = col.red
    g = col.green
    b = col.blue
    n = (r + g + b) / 3
    bwColorRGB = Color(n, n, n)
    return bwColorRGB


def HexColor(val, htmlOnly = False):
    '''This function converts a hex string, or an actual integer number,
    into the corresponding color.  E.g., in "#AABBCC" or 0xAABBCC,
    AA is the red, BB is the green, and CC is the blue (00-FF).

    For completeness I assume that #aabbcc or 0xaabbcc are hex numbers
    otherwise a pure integer is converted as decimal rgb.  If htmlOnly is true,
    only the #aabbcc form is allowed.

    >>> HexColor(\'#ffffff\')
    Color(1,1,1)
    >>> HexColor(\'#FFFFFF\')
    Color(1,1,1)
    >>> HexColor(\'0xffffff\')
    Color(1,1,1)
    >>> HexColor(\'16777215\')
    Color(1,1,1)

    An \'0x\' or \'#\' prefix is required for hex (as opposed to decimal):

    >>> HexColor(\'ffffff\')
    Traceback (most recent call last):
    ValueError: invalid literal for int(): ffffff

    >>> HexColor(\'#FFFFFF\', htmlOnly=True)
    Color(1,1,1)
    >>> HexColor(\'0xffffff\', htmlOnly=True)
    Traceback (most recent call last):
    ValueError: not a hex string
    >>> HexColor(\'16777215\', htmlOnly=True)
    Traceback (most recent call last):
    ValueError: not a hex string

    '''
    if type(val) == StringType:
        b = 10
        if val[:1] == '#':
            val = val[1:]
            b = 16
        elif htmlOnly:
            raise ValueError('not a hex string')
        
        if string.lower(val[:2]) == '0x':
            b = 16
            val = val[2:]
        
        val = string.atoi(val, b)
    
    return Color((val >> 16 & 255) / 255, (val >> 8 & 255) / 255, (val & 255) / 255)


def linearlyInterpolatedColor(c0, c1, x0, x1, x):
    """
    Linearly interpolates colors. Can handle RGB, CMYK and PCMYK
    colors - give ValueError if colours aren't the same.
    Doesn't currently handle 'Spot Color Interpolation'.
    """
    if c0.__class__ != c1.__class__:
        raise ValueError, 'Color classes must be the same for interpolation!'
    
    if x1 < x0:
        (x0, x1, c0, c1) = (x1, x0, c1, c0)
    
    if x < x0 - 1e-08 or x > x1 + 1e-08:
        raise ValueError, "Can't interpolate: x=%f is not between %f and %f!" % (x, x0, x1)
    
    if x <= x0:
        return c0
    elif x >= x1:
        return c1
    
    cname = c0.__class__.__name__
    dx = float(x1 - x0)
    x = x - x0
    if cname == 'Color':
        r = c0.red + x * (c1.red - c0.red) / dx
        g = c0.green + x * (c1.green - c0.green) / dx
        b = c0.blue + x * (c1.blue - c0.blue) / dx
        return Color(r, g, b)
    elif cname == 'CMYKColor':
        c = c0.cyan + x * (c1.cyan - c0.cyan) / dx
        m = c0.magenta + x * (c1.magenta - c0.magenta) / dx
        y = c0.yellow + x * (c1.yellow - c0.yellow) / dx
        k = c0.black + x * (c1.black - c0.black) / dx
        d = c0.density + x * (c1.density - c0.density) / dx
        return CMYKColor(c, m, y, k, density = d)
    elif cname == 'PCMYKColor':
        if cmykDistance(c0, c1) < 1e-08:
            if not c0.spotName == c1.spotName:
                raise AssertionError, 'Identical cmyk, but different spotName'
            c = c0.cyan
            m = c0.magenta
            y = c0.yellow
            k = c0.black
            d = c0.density + x * (c1.density - c0.density) / dx
            return PCMYKColor(c * 100, m * 100, y * 100, k * 100, density = d * 100, spotName = c0.spotName)
        elif cmykDistance(c0, _CMYK_white) < 1e-08:
            c = c1.cyan
            m = c1.magenta
            y = c1.yellow
            k = c1.black
            d = x * c1.density / dx
            return PCMYKColor(c * 100, m * 100, y * 100, k * 100, density = d * 100, spotName = c1.spotName)
        elif cmykDistance(c1, _CMYK_white) < 1e-08:
            c = c0.cyan
            m = c0.magenta
            y = c0.yellow
            k = c0.black
            d = x * c0.density / dx
            d = c0.density * (1 - x / dx)
            return PCMYKColor(c * 100, m * 100, y * 100, k * 100, density = d * 100, spotName = c0.spotName)
        else:
            c = c0.cyan + x * (c1.cyan - c0.cyan) / dx
            m = c0.magenta + x * (c1.magenta - c0.magenta) / dx
            y = c0.yellow + x * (c1.yellow - c0.yellow) / dx
            k = c0.black + x * (c1.black - c0.black) / dx
            d = c0.density + x * (c1.density - c0.density) / dx
            return PCMYKColor(c * 100, m * 100, y * 100, k * 100, density = d * 100)
    else:
        raise ValueError, "Can't interpolate: Unknown color class %s!" % cname

_CMYK_white = CMYKColor(0, 0, 0, 0)
_PCMYK_white = PCMYKColor(0, 0, 0, 0)
_CMYK_black = CMYKColor(0, 0, 0, 1)
_PCMYK_black = PCMYKColor(0, 0, 0, 100)
ReportLabBlueOLD = HexColor(5133960)
ReportLabBlue = HexColor(13183)
ReportLabBluePCMYK = PCMYKColor(100, 65, 0, 30, spotName = 'Pantone 288U')
ReportLabLightBlue = HexColor(12040659)
ReportLabFidBlue = HexColor(3368652)
ReportLabFidRed = HexColor(13369395)
ReportLabGreen = HexColor(3368448)
ReportLabLightGreen = HexColor(3381555)
aliceblue = HexColor(15792383)
antiquewhite = HexColor(16444375)
aqua = HexColor(65535)
aquamarine = HexColor(8388564)
azure = HexColor(15794175)
beige = HexColor(16119260)
bisque = HexColor(16770244)
black = HexColor(0)
blanchedalmond = HexColor(16772045)
blue = HexColor(255)
blueviolet = HexColor(9055202)
brown = HexColor(10824234)
burlywood = HexColor(14596231)
cadetblue = HexColor(6266528)
chartreuse = HexColor(8388352)
chocolate = HexColor(13789470)
coral = HexColor(16744272)
cornflowerblue = cornflower = HexColor(6591981)
cornsilk = HexColor(16775388)
crimson = HexColor(14423100)
cyan = HexColor(65535)
darkblue = HexColor(139)
darkcyan = HexColor(35723)
darkgoldenrod = HexColor(12092939)
darkgray = HexColor(11119017)
darkgreen = HexColor(25600)
darkkhaki = HexColor(12433259)
darkmagenta = HexColor(9109643)
darkolivegreen = HexColor(5597999)
darkorange = HexColor(16747520)
darkorchid = HexColor(10040012)
darkred = HexColor(9109504)
darksalmon = HexColor(15308410)
darkseagreen = HexColor(9419915)
darkslateblue = HexColor(4734347)
darkslategray = HexColor(3100495)
darkturquoise = HexColor(52945)
darkviolet = HexColor(9699539)
deeppink = HexColor(16716947)
deepskyblue = HexColor(49151)
dimgray = HexColor(6908265)
dodgerblue = HexColor(2003199)
firebrick = HexColor(11674146)
floralwhite = HexColor(16775920)
forestgreen = HexColor(2263842)
fuchsia = HexColor(16711935)
gainsboro = HexColor(14474460)
ghostwhite = HexColor(16316671)
gold = HexColor(16766720)
goldenrod = HexColor(14329120)
gray = HexColor(8421504)
grey = gray
green = HexColor(32768)
greenyellow = HexColor(11403055)
honeydew = HexColor(15794160)
hotpink = HexColor(16738740)
indianred = HexColor(13458524)
indigo = HexColor(4915330)
ivory = HexColor(16777200)
khaki = HexColor(15787660)
lavender = HexColor(15132410)
lavenderblush = HexColor(16773365)
lawngreen = HexColor(8190976)
lemonchiffon = HexColor(16775885)
lightblue = HexColor(11393254)
lightcoral = HexColor(15761536)
lightcyan = HexColor(14745599)
lightgoldenrodyellow = HexColor(16448210)
lightgreen = HexColor(9498256)
lightgrey = HexColor(13882323)
lightpink = HexColor(16758465)
lightsalmon = HexColor(16752762)
lightseagreen = HexColor(2142890)
lightskyblue = HexColor(8900346)
lightslategray = HexColor(7833753)
lightsteelblue = HexColor(11584734)
lightyellow = HexColor(16777184)
lime = HexColor(65280)
limegreen = HexColor(3329330)
linen = HexColor(16445670)
magenta = HexColor(16711935)
maroon = HexColor(8388608)
mediumaquamarine = HexColor(6737322)
mediumblue = HexColor(205)
mediumorchid = HexColor(12211667)
mediumpurple = HexColor(9662683)
mediumseagreen = HexColor(3978097)
mediumslateblue = HexColor(8087790)
mediumspringgreen = HexColor(64154)
mediumturquoise = HexColor(4772300)
mediumvioletred = HexColor(13047173)
midnightblue = HexColor(1644912)
mintcream = HexColor(16121850)
mistyrose = HexColor(16770273)
moccasin = HexColor(16770229)
navajowhite = HexColor(16768685)
navy = HexColor(128)
oldlace = HexColor(16643558)
olive = HexColor(8421376)
olivedrab = HexColor(7048739)
orange = HexColor(16753920)
orangered = HexColor(16729344)
orchid = HexColor(14315734)
palegoldenrod = HexColor(15657130)
palegreen = HexColor(10025880)
paleturquoise = HexColor(11529966)
palevioletred = HexColor(14381203)
papayawhip = HexColor(16773077)
peachpuff = HexColor(16767673)
peru = HexColor(13468991)
pink = HexColor(16761035)
plum = HexColor(14524637)
powderblue = HexColor(11591910)
purple = HexColor(8388736)
red = HexColor(16711680)
rosybrown = HexColor(12357519)
royalblue = HexColor(4286945)
saddlebrown = HexColor(9127187)
salmon = HexColor(16416882)
sandybrown = HexColor(16032864)
seagreen = HexColor(3050327)
seashell = HexColor(16774638)
sienna = HexColor(10506797)
silver = HexColor(12632256)
skyblue = HexColor(8900331)
slateblue = HexColor(6970061)
slategray = HexColor(7372944)
snow = HexColor(16775930)
springgreen = HexColor(65407)
steelblue = HexColor(4620980)
tan = HexColor(13808780)
teal = HexColor(32896)
thistle = HexColor(14204888)
tomato = HexColor(16737095)
turquoise = HexColor(4251856)
violet = HexColor(15631086)
wheat = HexColor(16113331)
white = HexColor(16777215)
whitesmoke = HexColor(16119285)
yellow = HexColor(16776960)
yellowgreen = HexColor(10145074)
fidblue = HexColor(3368652)
fidred = HexColor(13369395)
fidlightblue = HexColor('#d6e0f5')
ColorType = type(black)

def colorDistance(col1, col2):
    '''Returns a number between 0 and root(3) stating how similar
    two colours are - distance in r,g,b, space.  Only used to find
    names for things.'''
    return math.sqrt((col1.red - col2.red) ** 2 + (col1.green - col2.green) ** 2 + (col1.blue - col2.blue) ** 2)


def cmykDistance(col1, col2):
    '''Returns a number between 0 and root(4) stating how similar
    two colours are - distance in r,g,b, space.  Only used to find
    names for things.'''
    return math.sqrt((col1.cyan - col2.cyan) ** 2 + (col1.magenta - col2.magenta) ** 2 + (col1.yellow - col2.yellow) ** 2 + (col1.black - col2.black) ** 2)

_namedColors = None

def getAllNamedColors():
    global _namedColors
    if _namedColors is not None:
        return _namedColors
    
    import colors
    _namedColors = { }
    for name, value in colors.__dict__.items():
        if isinstance(value, Color):
            _namedColors[name] = value
            continue
    
    return _namedColors


def describe(aColor, mode = 0):
    '''finds nearest colour match to aColor.
    mode=0 print a string desription
    mode=1 return a string description
    mode=2 return (distance, colorName)
    '''
    namedColors = getAllNamedColors()
    closest = (10, None, None)
    for name, color in namedColors.items():
        distance = colorDistance(aColor, color)
        if distance < closest[0]:
            closest = (distance, name, color)
            continue
    
    if mode <= 1:
        s = 'best match is %s, distance %0.4f' % (closest[1], closest[0])
        if mode == 0:
            print s
        else:
            return s
    elif mode == 2:
        return (closest[1], closest[0])
    else:
        raise ValueError, 'Illegal value for mode ' + str(mode)


def toColor(arg, default = None):
    '''try to map an arbitrary arg to a color instance'''
    if isinstance(arg, Color):
        return arg
    
    tArg = type(arg)
    if tArg in _SeqTypes:
        if len(arg) <= len(arg):
            pass
        elif not len(arg) <= 4:
            raise AssertionError, 'Can only convert 3 and 4 sequences to color'
        if not 0 <= min(arg) or max(arg) <= 1:
            raise AssertionError
        if not len(arg) == 3 or Color(arg[0], arg[1], arg[2]):
            pass
        return CMYKColor(arg[0], arg[1], arg[2], arg[3])
    elif tArg == StringType:
        C = getAllNamedColors()
        s = string.lower(arg)
        if C.has_key(s):
            return C[s]
        
        
        try:
            return toColor(eval(arg))

    
    
    try:
        return HexColor(arg)
    except:
        if default is None:
            raise 'Invalid color value', str(arg)
        
        return default



def toColorOrNone(arg, default = None):
    '''as above but allows None as a legal value'''
    if arg is None:
        return None
    else:
        return toColor(arg, default)


def setColors(**kw):
    UNDEF = []
    progress = 1
    assigned = { }
    while kw and progress:
        progress = 0
        for k, v in kw.items():
            if type(v) in (type(()), type([])):
                c = map((lambda x, UNDEF = UNDEF: toColor(x, UNDEF)), v)
                if type(v) is type(()):
                    c = tuple(c)
                
                ok = UNDEF not in c
            else:
                c = toColor(v, UNDEF)
                ok = c is not UNDEF
            if ok:
                assigned[k] = c
                del kw[k]
                progress = 1
                continue
        
    if kw:
        raise ValueError("Can't convert\n%s" % str(kw))
    
    getAllNamedColors()
    for k, c in assigned.items():
        globals()[k] = c
        if isinstance(c, Color):
            _namedColors[k] = c
            continue
    


def Whiter(c, f):
    '''given a color combine with white as c*f w*(1-f) 0<=f<=1'''
    c = toColor(c)
    if isinstance(c, PCMYKColor):
        w = _PCMYK_white
    elif isinstance(c, CMYKColor):
        w = _CMYK_white
    else:
        w = white
    return linearlyInterpolatedColor(w, c, 0, 1, f)


def Blacker(c, f):
    '''given a color combine with black as c*f+b*(1-f) 0<=f<=1'''
    c = toColor(c)
    if isinstance(c, PCMYKColor):
        b = _PCMYK_black
    elif isinstance(c, CMYKColor):
        b = _CMYK_black
    else:
        b = black
    return linearlyInterpolatedColor(b, c, 0, 1, f)

if __name__ == '__main__':
    import doctest
    doctest.testmod()

