# Source Generated with Decompyle++
# File: in.pyc (Python 2.5)

__version__ = ' $Id: customshapes.py 2385 2004-06-17 15:26:05Z rgbecker $ '
__doc__ = '\nThis demonstrates a custom shape for use with the <customshape> tag.\nThe shape must fulfil a very simple interface, which may change in\nfuture.\n\nThe XML tag currently has this form:\n        <customshape\n            module="customshapes.py"\n            class = "MyShape"\n            initargs="(100,200,3)"\n        />\n\nPythonPoint will look in the given module for the given class,\nevaluate the arguments string and pass it to the constructor.\nThen, it will call\n\n    object.drawOn(canvas)\n\nThus your object must be fully defined by the constructor.\nFor this one, we pass three argumenyts: x, y and scale.\nThis does a five-tile jigsaw over which words can be overlaid;\nbased on work done for a customer\'s presentation.\n'
import reportlab.pdfgen.canvas as reportlab
from reportlab.lib import colors
from reportlab.lib.corp import RL_CorpLogo
from reportlab.graphics.shapes import Drawing

class Jigsaw:
    '''This draws a jigsaw patterm.  By default it is centred on 0,0
    and has dimensions of 200 x 140; use the x/y/scale attributes
    to move it around.'''
    
    def __init__(self, x, y, scale = 1):
        self.width = 200
        self.height = 140
        self.x = x
        self.y = y
        self.scale = scale

    
    def drawOn(self, canvas):
        canvas.saveState()
        canvas.setFont('Helvetica-Bold', 24)
        canvas.drawString(600, 100, 'A Custom Shape')
        canvas.translate(self.x, self.y)
        canvas.scale(self.scale, self.scale)
        self.drawBounds(canvas)
        self.drawCentre(canvas)
        self.drawTopLeft(canvas)
        self.drawBottomLeft(canvas)
        self.drawBottomRight(canvas)
        self.drawTopRight(canvas)
        canvas.restoreState()

    
    def curveThrough(self, path, pointlist):
        '''Helper to curve through set of control points.'''
        if not len(pointlist) % 3 == 1:
            raise AssertionError, 'No. of points must be 3n+1 for integer n'
        (x, y) = pointlist[0]
        path.moveTo(x, y)
        idx = 1
        while idx < len(pointlist) - 2:
            (p1, p2, p3) = pointlist[idx:idx + 3]
            path.curveTo(p1[0], p1[1], p2[0], p2[1], p3[0], p3[1])
            idx = idx + 3

    
    def drawShape(self, canvas, controls, color):
        '''Utlity to draw a closed shape through a list of control points;
        extends the previous proc'''
        canvas.setFillColor(color)
        p = canvas.beginPath()
        self.curveThrough(p, controls)
        p.close()
        canvas.drawPath(p, stroke = 1, fill = 1)

    
    def drawBounds(self, canvas):
        '''Guidelines to help me draw - not needed in production'''
        canvas.setStrokeColor(colors.red)
        canvas.rect(-100, -70, 200, 140)
        canvas.line(-100, 0, 100, 0)
        canvas.line(0, 70, 0, -70)
        canvas.setStrokeColor(colors.black)

    
    def drawCentre(self, canvas):
        controls = [
            (0, 50),
            (5, 50),
            (10, 45),
            (10, 40),
            (10, 35),
            (15, 30),
            (20, 30),
            (25, 30),
            (30, 25),
            (30, 20),
            (30, 15),
            (35, 10),
            (40, 10),
            (45, 10),
            (50, 5),
            (50, 0),
            (50, -5),
            (45, -10),
            (40, -10),
            (35, -10),
            (30, -15),
            (30, -20),
            (30, -25),
            (25, -30),
            (20, -30),
            (15, -30),
            (10, -35),
            (10, -40),
            (10, -45),
            (5, -50),
            (0, -50),
            (-5, -50),
            (-10, -45),
            (-10, -40),
            (-10, -35),
            (-15, -30),
            (-20, -30),
            (-25, -30),
            (-30, -25),
            (-30, -20),
            (-30, -15),
            (-35, -10),
            (-40, -10),
            (-45, -10),
            (-50, -5),
            (-50, 0),
            (-50, 5),
            (-45, 10),
            (-40, 10),
            (-35, 10),
            (-30, 15),
            (-30, 20),
            (-30, 25),
            (-25, 30),
            (-20, 30),
            (-15, 30),
            (-10, 35),
            (-10, 40),
            (-10, 45),
            (-5, 50),
            (0, 50)]
        self.drawShape(canvas, controls, colors.yellow)

    
    def drawTopLeft(self, canvas):
        controls = [
            (-100, 70),
            (-100, 69),
            (-100, 1),
            (-100, 0),
            (-99, 0),
            (-91, 0),
            (-90, 0),
            (-90, 5),
            (-92, 5),
            (-92, 10),
            (-92, 15),
            (-85, 15),
            (-80, 15),
            (-75, 15),
            (-68, 15),
            (-68, 10),
            (-68, 5),
            (-70, 5),
            (-70, 0),
            (-69, 0),
            (-51, 0),
            (-50, 0),
            (-50, 5),
            (-45, 10),
            (-40, 10),
            (-35, 10),
            (-30, 15),
            (-30, 20),
            (-30, 25),
            (-25, 30),
            (-20, 30),
            (-15, 30),
            (-10, 35),
            (-10, 40),
            (-10, 45),
            (-5, 50),
            (0, 50),
            (0, 51),
            (0, 69),
            (0, 70),
            (-1, 70),
            (-99, 70),
            (-100, 70)]
        self.drawShape(canvas, controls, colors.teal)

    
    def drawBottomLeft(self, canvas):
        controls = [
            (-100, -70),
            (-99, -70),
            (-1, -70),
            (0, -70),
            (0, -69),
            (0, -51),
            (0, -50),
            (-5, -50),
            (-10, -45),
            (-10, -40),
            (-10, -35),
            (-15, -30),
            (-20, -30),
            (-25, -30),
            (-30, -25),
            (-30, -20),
            (-30, -15),
            (-35, -10),
            (-40, -10),
            (-45, -10),
            (-50, -5),
            (-50, 0),
            (-51, 0),
            (-69, 0),
            (-70, 0),
            (-70, 5),
            (-68, 5),
            (-68, 10),
            (-68, 15),
            (-75, 15),
            (-80, 15),
            (-85, 15),
            (-92, 15),
            (-92, 10),
            (-92, 5),
            (-90, 5),
            (-90, 0),
            (-91, 0),
            (-99, 0),
            (-100, 0)]
        self.drawShape(canvas, controls, colors.green)

    
    def drawBottomRight(self, canvas):
        controls = [
            (100, -70),
            (100, -69),
            (100, -1),
            (100, 0),
            (99, 0),
            (91, 0),
            (90, 0),
            (90, -5),
            (92, -5),
            (92, -10),
            (92, -15),
            (85, -15),
            (80, -15),
            (75, -15),
            (68, -15),
            (68, -10),
            (68, -5),
            (70, -5),
            (70, 0),
            (69, 0),
            (51, 0),
            (50, 0),
            (50, -5),
            (45, -10),
            (40, -10),
            (35, -10),
            (30, -15),
            (30, -20),
            (30, -25),
            (25, -30),
            (20, -30),
            (15, -30),
            (10, -35),
            (10, -40),
            (10, -45),
            (5, -50),
            (0, -50),
            (0, -51),
            (0, -69),
            (0, -70),
            (1, -70),
            (99, -70),
            (100, -70)]
        self.drawShape(canvas, controls, colors.navy)

    
    def drawBottomLeft(self, canvas):
        controls = [
            (-100, -70),
            (-99, -70),
            (-1, -70),
            (0, -70),
            (0, -69),
            (0, -51),
            (0, -50),
            (-5, -50),
            (-10, -45),
            (-10, -40),
            (-10, -35),
            (-15, -30),
            (-20, -30),
            (-25, -30),
            (-30, -25),
            (-30, -20),
            (-30, -15),
            (-35, -10),
            (-40, -10),
            (-45, -10),
            (-50, -5),
            (-50, 0),
            (-51, 0),
            (-69, 0),
            (-70, 0),
            (-70, 5),
            (-68, 5),
            (-68, 10),
            (-68, 15),
            (-75, 15),
            (-80, 15),
            (-85, 15),
            (-92, 15),
            (-92, 10),
            (-92, 5),
            (-90, 5),
            (-90, 0),
            (-91, 0),
            (-99, 0),
            (-100, 0)]
        self.drawShape(canvas, controls, colors.green)

    
    def drawTopRight(self, canvas):
        controls = [
            (100, 70),
            (99, 70),
            (1, 70),
            (0, 70),
            (0, 69),
            (0, 51),
            (0, 50),
            (5, 50),
            (10, 45),
            (10, 40),
            (10, 35),
            (15, 30),
            (20, 30),
            (25, 30),
            (30, 25),
            (30, 20),
            (30, 15),
            (35, 10),
            (40, 10),
            (45, 10),
            (50, 5),
            (50, 0),
            (51, 0),
            (69, 0),
            (70, 0),
            (70, -5),
            (68, -5),
            (68, -10),
            (68, -15),
            (75, -15),
            (80, -15),
            (85, -15),
            (92, -15),
            (92, -10),
            (92, -5),
            (90, -5),
            (90, 0),
            (91, 0),
            (99, 0),
            (100, 0)]
        self.drawShape(canvas, controls, colors.magenta)



class Logo:
    '''This draws a ReportLab Logo.'''
    
    def __init__(self, x, y, width, height):
        logo = RL_CorpLogo()
        logo.x = x
        logo.y = y
        logo.width = width
        logo.height = height
        self.logo = logo

    
    def drawOn(self, canvas):
        logo = self.logo
        x = logo.x
        y = logo.y
        w = logo.width
        h = logo.height
        D = Drawing(w, h)
        D.add(logo)
        D.drawOn(canvas, 0, 0)



def run():
    c = reportlab.pdfgen.canvas.Canvas('customshape.pdf')
    J = Jigsaw(300, 540, 2)
    J.drawOn(c)
    c.save()

if __name__ == '__main__':
    run()

